from __future__ import annotations
from typing import Optional, Union

from PySide6.QtCore import QPointF, Qt, QRectF
from PySide6.QtWidgets import QWidget, QVBoxLayout, QGraphicsItem
from PySide6.QtCharts import (
    QChart, QChartView,
    QSplineSeries, QAreaSeries,
    QBarCategoryAxis, QValueAxis,
    QScatterSeries, QLineSeries
)
from PySide6.QtGui import (
    QPainter, QColor, QFont, QPen, QBrush, QFontMetrics, QLinearGradient, QGradient
)

from qtmui.material.styles import useTheme


# -------------------------------------------------------
# TooltipItem used by ChartArea
# -------------------------------------------------------
class TooltipItem(QGraphicsItem):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.lines = []
        self.padding = 8
        self.bg_color = QColor(20, 20, 20, 230)
        self.border_color = QColor(255, 255, 255, 90)
        self.border_radius = 6
        self.font = QFont("Segoe UI", 9)

        self._w = 0
        self._h = 0
        self.setZValue(99999)
        self.hide()

    def clear(self):
        self.lines = []
        self._w = 0
        self._h = 0
        self.hide()

    def add_line(self, text: str, color: QColor):
        self.lines.append((text, color))
        fm = QFontMetrics(self.font)
        # width includes marker (8px) + spacing
        w = fm.horizontalAdvance(text) + 8 + 6
        h = fm.height()
        self._w = max(self._w, w)
        self._h += h

    def boundingRect(self) -> QRectF:
        return QRectF(0, 0,
                      self._w + self.padding * 2,
                      self._h + self.padding * 2)

    def paint(self, painter: QPainter, option, widget=None):
        if not self.lines:
            return

        painter.setRenderHint(QPainter.Antialiasing)
        painter.setFont(self.font)
        fm = QFontMetrics(self.font)

        rect = self.boundingRect()
        painter.setBrush(QBrush(self.bg_color))
        painter.setPen(QPen(self.border_color, 1))
        painter.drawRoundedRect(rect, self.border_radius, self.border_radius)

        y = self.padding
        for text, color in self.lines:
            # marker circle
            painter.setBrush(color)
            painter.setPen(Qt.NoPen)
            painter.drawEllipse(self.padding, y + fm.height() / 2 - 4, 8, 8)
            # text
            painter.setPen(QPen(QColor("white")))
            painter.drawText(self.padding + 14, y + fm.ascent(), text)
            y += fm.height()


# -------------------------------------------------------
# HoverChartView (to forward events to owner)
# -------------------------------------------------------
class HoverChartView(QChartView):
    def __init__(self, chart, parent=None):
        super().__init__(chart, parent)
        self.owner: 'ChartArea' = None
        self.setMouseTracking(True)

    def mouseMoveEvent(self, event):
        if self.owner:
            self.owner.handle_hover(event)
        super().mouseMoveEvent(event)

    def leaveEvent(self, event):
        if self.owner:
            self.owner.hide_hover()
        super().leaveEvent(event)


# -------------------------------------------------------
# ChartArea: QSplineSeries + QAreaSeries, tooltip, hover
# -------------------------------------------------------
class ChartArea(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        type: str = "area",
        series: object = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: object = None,
        key: str = None,
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        # normalize series input
        self._series_input = series or []  # expect list[{"name": str, "data": [numbers]}]
        self._options = options or {}
        self._width = width
        self._height = height

        self._init_area_chart()

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _init_area_chart(self):
        # sizing
        if self._height:
            self.setFixedHeight(self._height)
        if self._width:
            self.setFixedWidth(self._width)

        # Normalize and collect all data arrays, convert to floats
        all_series_data = []
        for item in self._series_input:
            if isinstance(item, dict):
                data = item.get("data", []) or []
            else:
                data = item or []
            all_series_data.append([float(v) for v in data])

        # Compute global max across all series (fix for your issue)
        max_val = max((max(d) for d in all_series_data), default=0)

        # Build spline series (QSplineSeries) and separate visible line series for each input
        # We'll create:
        #   - upper_spline (QSplineSeries) used as upper series for area (smoothed)
        #   - line_spline  (QSplineSeries) used to draw the stroke (also smoothed)
        self._spline_list = []       # visible line splines
        self._upper_list = []        # splines used as upper for area
        self._area_items = []        # tuples (upper, baseline, area, line)

        for idx, item in enumerate(self._series_input):
            name = item.get("name", "") if isinstance(item, dict) else ""
            data = all_series_data[idx]

            # create smoothed upper spline (for area)
            upper = QSplineSeries()
            for i, y in enumerate(data):
                upper.append(QPointF(float(i), float(y)))

            # create line spline (for stroke). Use a separate object so we can control legend names.
            line = QSplineSeries()
            for i, y in enumerate(data):
                line.append(QPointF(float(i), float(y)))
            line.setName("")  # avoid duplicate legend entries; area will carry the name

            # create baseline spline (y=0) with same x coordinates
            baseline = QSplineSeries()
            for i in range(upper.count()):
                pt = upper.at(i)
                baseline.append(QPointF(pt.x(), 0.0))

            # create area from upper and baseline
            area = QAreaSeries(upper, baseline)
            area.setName(name)  # area shown in legend

            # collect and add to chart
            self._spline_list.append(line)
            self._upper_list.append(upper)
            self._area_items.append((upper, baseline, area, line))

        # Create chart and add series (areas and lines)
        self.chart = QChart()
        self.chart.legend().setVisible(True)
        self.chart.legend().setAlignment(Qt.AlignTop)

        for upper, baseline, area, line in self._area_items:
            # add area first (so legend shows area), then line for stroke
            self.chart.addSeries(area)
            self.chart.addSeries(line)

        # Axes
        categories = self._options.get("xaxis", {}).get("categories", None)
        if categories:
            from PySide6.QtCharts import QBarCategoryAxis
            self.axis_x = QBarCategoryAxis()
            self.axis_x.append(categories)
            self.axis_y = QValueAxis()
            self.axis_y.setRange(0, max_val)
        else:
            self.axis_x = QValueAxis()
            self.axis_y = QValueAxis()
            self.axis_y.setRange(0, max_val)

        self.chart.addAxis(self.axis_x, Qt.AlignBottom)
        self.chart.addAxis(self.axis_y, Qt.AlignLeft)

        # Attach axes for each spline and area
        for upper, baseline, area, line in self._area_items:
            # each element needs to be attached to axes for correct mapping
            upper.attachAxis(self.axis_x)
            upper.attachAxis(self.axis_y)
            area.attachAxis(self.axis_x)
            area.attachAxis(self.axis_y)
            line.attachAxis(self.axis_x)
            line.attachAxis(self.axis_y)
            # baseline is internal, not attached to chart

        # Hover markers (one per input series)
        self.hover_markers = []
        for idx, line in enumerate(self._spline_list):
            mk = QScatterSeries()
            mk.setMarkerShape(QScatterSeries.MarkerShapeCircle)
            mk.setMarkerSize(10)
            # initial color use line's pen color (will sync in theme)
            mk.setColor(QColor("#00e5ff"))
            mk.setBorderColor(QColor("#ffffff"))
            mk.hide()
            self.chart.addSeries(mk)
            mk.attachAxis(self.axis_x)
            mk.attachAxis(self.axis_y)
            self.hover_markers.append(mk)

        # Vertical hover line
        self.hover_line = QLineSeries()
        self.hover_line.setPen(QPen(QColor("#8888ff"), 1, Qt.DashLine))
        self.hover_line.hide()
        self.chart.addSeries(self.hover_line)
        self.hover_line.attachAxis(self.axis_x)
        self.hover_line.attachAxis(self.axis_y)

        # Tooltip item (custom QGraphicsItem)
        self.tooltip = TooltipItem(self.chart)
        self.tooltip.hide()

        # ChartView (use custom to forward mouse events)
        self.chart_view = HoverChartView(self.chart)
        self.chart_view.owner = self
        self.chart_view.setRenderHint(QPainter.Antialiasing)

        # Add tooltip into the view's scene
        scene = self.chart_view.scene()
        if scene:
            scene.addItem(self.tooltip)

        self.layout().addWidget(self.chart_view)

    def hide_hover(self):
        # hide tooltip, markers, line
        self.tooltip.clear()
        self.tooltip.hide()
        self.hover_line.hide()
        for mk in self.hover_markers:
            mk.hide()

    def handle_hover(self, event):
        if not self._spline_list:
            return

        pos = event.pos()
        # use first line (spline) to map viewport pos -> value coords
        reference_series = self._spline_list[0] if self._spline_list else None
        if reference_series is None:
            return

        chart_pos = self.chart.mapToValue(pos, reference_series)
        x_float = chart_pos.x()
        # nearest integer index
        nearest_idx = round(x_float)
        # clamp index within 0..count-1 (use first upper to determine count)
        total_points = self._area_items[0][0].count() if self._area_items else 0
        nearest_idx = max(0, min(nearest_idx, max(0, total_points - 1)))

        # collect points for each upper/list
        pts = []
        for upper, baseline, area, line in self._area_items:
            if nearest_idx < upper.count():
                pts.append(upper.at(nearest_idx))
            else:
                pts.append(QPointF(nearest_idx, 0.0))

        # update hover markers and sync their colors to corresponding line/area
        for i, mk in enumerate(self.hover_markers):
            mk.clear()
            mk.append(pts[i])
            # sync color: use pen color of visible line (which we style later in theme)
            color = self._spline_list[i].pen().color() if i < len(self._spline_list) else QColor("#00e5ff")
            mk.setColor(color)
            mk.show()

        # update vertical line
        ymin, ymax = self.axis_y.min(), self.axis_y.max()
        self.hover_line.clear()
        self.hover_line.append(nearest_idx, ymin)
        self.hover_line.append(nearest_idx, ymax)
        self.hover_line.show()

        # tooltip content
        self.tooltip.clear()
        categories = self._options.get("xaxis", {}).get("categories")
        title = categories[nearest_idx] if categories and nearest_idx < len(categories) else f"Index {nearest_idx}"
        self.tooltip.add_line(title, QColor("#cccccc"))

        for idx, (upper, baseline, area, line) in enumerate(self._area_items):
            # get display name from area.name() (we set it earlier)
            display_name = area.name() or (self._series_input[idx].get("name") if isinstance(self._series_input[idx], dict) else "")
            value_text = f"{display_name}: {pts[idx].y():.2f}"
            color = line.pen().color()  # color of line (we'll style it in theme)
            self.tooltip.add_line(value_text, color)

        # prepare geometry
        self.tooltip.prepareGeometryChange()

        # position tooltip near the vertical line: use mapToPosition to convert a value to pixel pos
        ref = self.chart.mapToPosition(QPointF(nearest_idx, ymax), reference_series)
        plot_area = self.chart.plotArea()
        center_x = plot_area.left() + plot_area.width() / 2

        rect = self.tooltip.boundingRect()
        w = rect.width()
        # place to right or left depending on which half
        if ref.x() < center_x:
            tx = ref.x() + 15
        else:
            tx = ref.x() - w - 20
        ty = ref.y() + 10

        self.tooltip.setPos(tx, ty)
        self.tooltip.show()

    def _set_stylesheet(self):
        # unify style with your SplineChart implementation
        theme = useTheme()

        text_color = QColor(theme.palette.text.secondary)
        primary_color = QColor(theme.palette.primary.main)

        self.chart.setTitleBrush(text_color)

        font = QFont()
        font.setPointSize(8)
        font.setWeight(QFont.Weight(600))

        pen = QPen(Qt.DashLine)
        pen.setWidthF(0.2)
        pen.setColor(text_color)

        # X axis style
        try:
            self.axis_x.setGridLineVisible(False)
            self.axis_x.setLabelsBrush(text_color)
            self.axis_x.setLinePen(pen)
            self.axis_x.setLabelsFont(font)
        except Exception:
            pass

        # Y axis style
        try:
            self.axis_y.setLinePen(Qt.NoPen)
            self.axis_y.setGridLinePen(pen)
            self.axis_y.setLabelsFont(font)
        except Exception:
            pass

        legend = self.chart.legend()
        legend.setLabelBrush(text_color)

    # style area and line series consistently
        for idx, (upper, baseline, area, line) in enumerate(self._area_items):
            color = primary_color if idx == 0 else QColor(theme.palette.info.main)

            # 1. Đường viền mượt: dùng line (QSplineSeries) để vẽ stroke
            pen_line = QPen(color)
            pen_line.setWidth(3)                    # độ dày viền
            pen_line.setCapStyle(Qt.RoundCap)       # bo tròn đầu cuối (đẹp hơn)
            pen_line.setJoinStyle(Qt.RoundJoin)     # bo góc khi dữ liệu gấp khúc
            line.setPen(pen_line)

            # 2. Ẩn hoàn toàn đường viền của upper series (không cần thiết)
            upper.setPen(QPen(Qt.NoPen))            # hoặc width=0

            # 3. QUAN TRỌNG: Tắt hoàn toàn border của QAreaSeries
            area.setPen(Qt.NoPen)  # nếu để width > 0 thì Qt sẽ vẽ border zigzag!!!

            # Fill area bằng gradient (giữ nguyên)
            grad = QLinearGradient(0, 0, 0, 1)
            grad.setCoordinateMode(QGradient.ObjectBoundingMode)
            grad.setColorAt(0.0, color.lighter(140))
            grad.setColorAt(1.0, color.darker(180))
            area.setBrush(QBrush(grad))

        # sync hover marker colors with line pens
        for i, mk in enumerate(self.hover_markers):
            if i < len(self._spline_list):
                mk.setColor(self._spline_list[i].pen().color())
                mk.setBorderColor(QColor("#ffffff"))



from typing import Callable, Dict, Optional, Union
from PySide6.QtWebEngineWidgets import QWebEngineView
from PySide6.QtCore import QUrl, QObject, Slot, QMarginsF, QTimer
from PySide6.QtGui import QAction, QPageLayout, QPageSize
from PySide6.QtWebChannel import QWebChannel
import markdown
import base64, urllib.parse
from PySide6.QtWidgets import QMenu, QApplication, QFileDialog, QMessageBox
import webbrowser, copy
from enum import Enum
from qtmui.hooks import State

# from src.components.site_packages.markdown.extensions.tables


__all__ = ["MarkdownView", "LinkMiddlewarePolicy"]
aboutInformation = """Markdown 0.2
a package based on ..site_packages.qtcompat designed to help you preview Markdown documents.
"""

class LinkMiddlewarePolicy(Enum):
    OpenNewTab = 0
    Open = 1
    OpenNew = 2


class Markdown(QWebEngineView):
    extensions = None
    value = ""

    class LinkMiddleware(QObject):
        policy = None

        @Slot(str)
        def open_external(self, url):
            if url is None:
                return
            if self.policy == LinkMiddlewarePolicy.Open:
                webbrowser.open(url)
            if self.policy == LinkMiddlewarePolicy.OpenNew:
                webbrowser.open_new(url)
            else:
                webbrowser.open_new_tab(url)

    def __init__(
                self, 
                parent=None,
                content: str = None,
                sx: Optional[Union[Callable, str, Dict]]= None,
                ):
        super().__init__(parent=parent)
        self.load(QUrl("qrc:/markdown_view/page.html"))
        self.channel = QWebChannel()
        self.page().setWebChannel(self.channel)
        self.link_middleware = self.LinkMiddleware()
        self.extensions = list()
        self.channel.registerObject("link_middleware", self.link_middleware)
        self.setLinkMiddlewarePolicy(LinkMiddlewarePolicy.OpenNewTab)

        # Sang Pc
        self.setExtensions(["markdown.extensions.tables", "markdown.extensions.extra"])
        self.setLinkMiddlewarePolicy(LinkMiddlewarePolicy.OpenNew)

        if content:
            if isinstance(content, State):
                content.valueChanged.connect(self.setValue)
                self.setValue(content.value)
            elif isinstance(content, str):
                self.setValue(content)


    def setValue(self, value: str):
        self.value = value
        extensions = self.getExtensions()
        body_html = markdown.markdown(value, extensions=extensions)
        bs64 = base64.b64encode(urllib.parse.quote(body_html).encode()).decode()
        self.script = "setValue({});".format(repr(bs64))
        QTimer.singleShot(1000, self._run_script)

    def _run_script(self):
        self.page().runJavaScript(self.script)

    def getValue(self) -> str:
        return self.value

    def contextMenuEvent(self, arg__1) -> None:
        menu = QMenu(self)
        reload_page = QAction("Reload")
        reload_page.triggered.connect(lambda : self.setValue(self.getValue()))
        copy_page = QAction("Copy Source Code")
        def Copy():
            QApplication.clipboard().setText(self.getValue())
            QMessageBox.information(self, "Markdown View", "Source code has been copied to clipboard.")
        copy_page.triggered.connect(Copy)
        pdf_export = QAction("Export to PDF")
        def pdfExport():
            fp = QFileDialog.getSaveFileName(self, "Save PDF...", filter = "PDF(*.pdf)")[0]
            if fp is None or fp == "":
                return
            lay = QPageLayout(QPageSize(QPageSize.PageSizeId.A4), QPageLayout.Orientation.Portrait, QMarginsF())
            self.page().printToPdf(fp, lay)
            QMessageBox.information(self, "Markdown View", "The document has been exported successfully.")
        pdf_export.triggered.connect(pdfExport)
        about = QAction("About")
        about.triggered.connect(lambda : QMessageBox.information(self, "Markdown View", aboutInformation))
        menu.addActions([reload_page, copy_page, pdf_export, about])
        menu.exec(arg__1.globalPos())

    def setLinkMiddlewarePolicy(self, policy) -> None:
        self.link_middleware.policy = policy

    def getLinkMiddlewarePolicy(self):
        return self.link_middleware.policy

    def setExtensions(self, extensions: list[str]) -> None:
        self.extensions = copy.deepcopy(extensions)

    def getExtensions(self) -> list[str]:
        return copy.deepcopy(self.extensions)

    def addExtension(self, extension: str) -> None:
        self.extensions.append(extension)



from typing import TYPE_CHECKING
from ...system.color_manipulator import alpha
# from PySide6.QtSvgWidgets import QSvgWidget

if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState
    

# class ArrowDownIcon(QSvgWidget):
#     def __init__(self, parent=None):
#         super().__init__(parent)
#         svg_data = b"""
#             <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24">
#                 <path d="M12,16 C11.7663478,16.0004565 11.5399121,15.9190812 11.36,15.77 L5.36,10.77 
#                 C4.93474074,10.4165378 4.87653776,9.78525926 5.23,9.36 C5.58346224,8.93474074 
#                 6.21474074,8.87653776 6.64,9.23 L12,13.71 L17.36,9.39 C17.5665934,9.2222295 
#                 17.8315409,9.14373108 18.0961825,9.17188444 C18.3608241,9.2000378 18.6033268,9.33252029 
#                 18.77,9.54 C18.9551341,9.74785947 19.0452548,10.0234772 19.0186853,10.3005589 
#                 C18.9921158,10.5776405 18.8512608,10.8311099 18.63,11 L12.63,15.83 
#                 C12.444916,15.955516 12.2231011,16.0153708 12,16 Z" />
#             </svg>
#         """
#         self.load(svg_data)
#         self.setFixedSize(24, 24)


# class CheckboxIcon(QSvgWidget):
#     def __init__(self, parent=None):
#         super().__init__(parent)
#         svg_data = b"""
#             <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24">
#                 <path d="M17.9 2.318A5 5 0 0 1 22.895 7.1l.005.217v10a5 5 0 0 1-4.783 4.995l-.217.005h-10a5 5 0 0 1-4.995-4.783l-.005-.217v-10a5 5 0 0 1 4.783-4.996l.217-.004h10Zm-.5 1.5h-9a4 4 0 0 0-4 4v9a4 4 0 0 0 4 4h9a4 4 0 0 0 4-4v-9a4 4 0 0 0-4-4Z" />
#             </svg>
#         """
#         self.load(svg_data)
#         self.setFixedSize(24, 24)


# class CheckboxCheckedIcon(QSvgWidget):
#     def __init__(self, parent=None):
#         super().__init__(parent)
#         svg_data = b"""
#             <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24">
#                 <path d="M17 2a5 5 0 0 1 5 5v10a5 5 0 0 1-5 5H7a5 5 0 0 1-5-5V7a5 5 0 0 1 5-5Zm-1.625 7.255-4.13 4.13-1.75-1.75a.881.881 0 0 0-1.24 0c-.34.34-.34.89 0 1.24l2.38 2.37c.17.17.39.25.61.25.23 0 .45-.08.62-.25l4.75-4.75c.34-.34.34-.89 0-1.24a.881.881 0 0 0-1.24 0Z" />
#             </svg>
#         """
#         self.load(svg_data)
#         self.setFixedSize(24, 24)

# Các biểu tượng khác tương tự như vậy...

class Iconify:
    icon: str
    width: int
    def __init__(self, icon=None, width=None):
        pass


def default_props(_theme):
    theme: ThemeState = _theme
    return {
        "PyAlert": {
            "defaultProps": {
                "iconMapping": {
                    "error": 'Iconify(icon="solar:danger-bold", width=24)',
                    "info": 'Iconify(icon="eva:info-fill", width=24)',
                    "success": 'Iconify(icon="eva:checkmark-circle-2-fill", width=24)',
                    "warning": 'Iconify(icon="eva:alert-triangle-fill", width=24)',
                },
            },
        },
        "PyStack": {
            "defaultProps": {
                "useFlexGap": True,
            },
        },
        "PyAppBar": {
            "defaultProps": {
                "color": "transparent",
            },
        },
        "PyAvatarGroup": {
            "defaultProps": {
                "max": 4,
            },
        },
        "MuiButtonGroup": {
            "defaultProps": {
                "disableElevation": True,
            },
        },
        "MuiButton": {
            "defaultProps": {
                "color": "inherit",
                "disableElevation": True,
            },
        },
        "PyCardHeader": {
            "defaultProps": {
                "titleTypographyProps": {"variant": "h6"},
                "subheaderTypographyProps": {
                    "variant": "body2",
                    "marginTop": theme.spacing(0.5),
                },
            },
        },
        "PyChip": {
            "defaultProps": {
                "deleteIcon": 'Iconify(icon="solar:close-circle-bold")',
            },
        },
        "PyDialogActions": {
            "defaultProps": {
                "disableSpacing": True,
            },
        },
        "PyFab": {
            "defaultProps": {
                "color": "primary",
            },
        },
        "PyLink": {
            "defaultProps": {
                "underline": "hover",
            },
        },
        "PyListItemText": {
            "defaultProps": {
                "primaryTypographyProps": {
                    "typography": "subtitle2",
                },
                "secondaryTypographyProps": {
                    "component": "span",
                },
            },
        },
        "PyPaper": {
            "defaultProps": {
                "elevation": 0,
                "background-color": theme.palette.background.neutral
            },
        },
        "PySkeleton": {
            "defaultProps": {
                "animation": "wave",
                "variant": "rounded",
            },
        },
        "MuiFilledInput": {
            "defaultProps": {
                "disableUnderline": True,
            },
        },
        "MuiFormHelperText": {
            "defaultProps": {
                "component": "div",
            },
        },
        "PyTab": {
            "defaultProps": {
                "disableRipple": True,
                "iconPosition": "start",
            },
        },
        "PyTabs": {
            "defaultProps": {
                "textColor": "inherit",
                "variant": "scrollable",
                "allowScrollButtonsMobile": True,
            },
        },
        "PyTablePagination": {
            "defaultProps": {
                "backIconButtonProps": {
                    "size": "small",
                },
                "nextIconButtonProps": {
                    "size": "small",
                },
            },
        },
        "PySlider": {
            "defaultProps": {
                "size": "small",
            },
        },
        "MuiAutocomplete": {
            "defaultProps": {
                "popupIcon": 'ArrowDownIcon()',
            },
        },
        "PySelect": {
            "defaultProps": {
                "IconComponent": 'ArrowDownIcon()',
            },
        },
        "PyNativeSelect": {
            "defaultProps": {
                "IconComponent": 'ArrowDownIcon()',
            },
        },
        "MuiCheckbox": {
            "defaultProps": {
                "size": "small",
                "icon": 'CheckboxIcon()',
                "checkedIcon": 'CheckboxCheckedIcon()',
                "indeterminateIcon": 'CheckboxCheckedIcon()',  # Icon tương tự như checked trong trường hợp này
            },
        },
        "PyRadio": {
            "defaultProps": {
                "size": "small",
                "icon": 'CheckboxIcon()',
                "checkedIcon": 'CheckboxCheckedIcon()',
            },
        },
        "PyRating": {
            "defaultProps": {
                "emptyIcon": 'CheckboxIcon()',
                "icon": 'CheckboxCheckedIcon()',
            },
        },
        "PyTreeView": {
            "defaultProps": {
                "defaultCollapseIcon": 'CheckboxIcon()',
                "defaultExpandIcon": 'CheckboxCheckedIcon()',
                "defaultEndIcon": 'CheckboxCheckedIcon()',
            },
        },
    }



def create_nav_data(data):
    # Khởi tạo cấu trúc kết quả với subheader
    result = [{
        'subheader': 'Members',
        'items': []
    }]
    
    # Tạo từ điển để theo dõi các node đã xử lý
    nodes = {}
    
    # Tạo tất cả các node và thêm vào dictionary nodes
    for item in data:
        node = {
            'title': item['title'],
            "parent_department": item['parent'],
            'path': '#',
            'icon': 'None',
            'id': item['id'],
            'onActionButtonClicked': 'None',
            'children': []
        }
        nodes[item['id']] = node

    # Thêm các node vào cây theo thứ tự parent-child
    for item in data:
        node = nodes[item['id']]
        if item['parent'] is None:
            result[0]['items'].append(node)
        else:
            parent_node = nodes.get(item['parent'])
            if parent_node:
                parent_node['children'].append(node)
    
    return result


import asyncio
from typing import Optional, Union, Callable, Any, Dict, List
import uuid

from qtmui.hooks import State, useEffect
from PySide6.QtWidgets import QPushButton
from PySide6.QtGui import Qt, QPixmap, QPainter, QPen, QBrush, QColor, QFont
from PySide6.QtCore import Qt, QRect, QSize, QByteArray, QTimer
from PySide6.QtSvg import QSvgRenderer

from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.styles.create_theme.create_palette import PaletteColor

from ..system.color_manipulator import alpha
from ...common.icon import writeSvg
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from ..py_tool_button.py_tool_button import PyToolButton
from ..widget_base import PyWidgetBase

from ...qtmui_assets import QTMUI_ASSETS

from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n
from ..utils.validate_params import _validate_param

class Badge(QPushButton, PyWidgetBase):
    """
    A component that displays a badge relative to a child node, with customizable content and positioning.

    The `Badge` component is used to highlight additional information, such as notifications or status,
    relative to a child component. It supports all props of the Material-UI `Badge` component, including
    additional props for icons, keys, and text. Native component props are supported via `**kwargs`.

    Parameters
    ----------
    anchorOrigin : State or dict, optional
        The anchor position of the badge. Must contain `vertical` ("top" or "bottom") and
        `horizontal` ("left" or "right"). Default is {"vertical": "top", "horizontal": "right"}.
        Can be a `State` object for dynamic updates.
    badgeContent : State or Any, optional
        The content rendered within the badge (e.g., text, number, or component). Default is None.
        Can be a `State` object for dynamic updates.
    children : State or Any, optional
        The node relative to which the badge is added. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    color : State or str, optional
        The color of the badge. Valid values: "default", "primary", "secondary", "error", "info",
        "success", "warning". Default is "default". Can be a `State` object for dynamic updates.
    component : State or Any, optional
        Component used for the root node (e.g., HTML element or custom component).
        Default is None. Can be a `State` object for dynamic updates.
    components : State or dict, optional
        Components used for each slot (e.g., Badge, Root). Deprecated: Use `slots` instead.
        Default is None. Can be a `State` object for dynamic updates.
    componentsProps : State or dict, optional
        Extra props for slot components (e.g., badge, root). Deprecated: Use `slotProps` instead.
        Default is None. Can be a `State` object for dynamic updates.
    invisible : State or bool, optional
        If True, the badge is invisible. Default is False.
        Can be a `State` object for dynamic updates.
    icon : State or str, optional
        Path to an SVG icon to display in the badge. Default is None.
        Can be a `State` object for dynamic updates.
    max : State or int, optional
        Maximum count to show in the badge. Default is 99.
        Can be a `State` object for dynamic updates.
    overlap : State or str, optional
        Shape the badge overlaps. Valid values: "circular", "rectangular". Default is "rectangular".
        Can be a `State` object for dynamic updates.
    key : State or str, optional
        Unique key for the component, used for internal purposes. Default is None.
        Can be a `State` object for dynamic updates.
    showZero : State or bool, optional
        If True, displays the badge when `badgeContent` is 0. Default is False.
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (e.g., badge, root). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components for each slot (e.g., badge, root). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        Variant of the badge. Valid values: "dot", "standard". Default is "standard".
        Can be a `State` object for dynamic updates.
    text : State or str, optional
        Text to display in the badge. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QPushButton` class, supporting
        native component props (e.g., style, className).

    Attributes
    ----------
    VALID_VARIANTS : list[str]
        Valid values for the `variant` parameter: ["dot", "standard"].
    VALID_COLORS : list[str]
        Valid values for the `color` parameter: ["default", "primary", "secondary", "error",
        "info", "success", "warning"].
    VALID_OVERLAP : list[str]
        Valid values for the `overlap` parameter: ["circular", "rectangular"].
    VALID_ANCHOR_VERTICAL : list[str]
        Valid values for `anchorOrigin.vertical`: ["top", "bottom"].
    VALID_ANCHOR_HORIZONTAL : list[str]
        Valid values for `anchorOrigin.horizontal`: ["left", "right"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `components` and `componentsProps` props are deprecated; use `slots` and `slotProps` instead.
    - Additional props (`icon`, `key`, `text`) are specific to this implementation and not part of
      Material-UI `Badge`.

    Demos:
    - Badge: https://qtmui.com/material-ui/qtmui-badge/

    API Reference:
    - Badge API: https://qtmui.com/material-ui/api/badge/
    """

    VALID_VARIANTS = ["dot", "standard"]
    VALID_COLORS = ["default", "primary", "secondary", "error", "info", "success", "warning"]
    VALID_OVERLAP = ["circular", "rectangular"]
    VALID_ANCHOR_VERTICAL = ["top", "bottom"]
    VALID_ANCHOR_HORIZONTAL = ["left", "right"]

    def __init__(
        self,
        anchorOrigin: Union[State, Dict[str, str]] = None,
        badgeContent: Optional[Union[State, Any]] = None,
        children: Optional[Union[State, Any]] = None,
        classes: Optional[Union[State, Dict]] = None,
        color: Union[State, str] = "default",
        component: Optional[Union[State, Any]] = None,
        components: Optional[Union[State, Dict]] = None,
        componentsProps: Optional[Union[State, Dict]] = None,
        invisible: Union[State, bool] = False,
        icon: Optional[Union[State, str]] = None,
        max: Union[State, int] = 99,
        overlap: Union[State, str] = "rectangular",
        key: Optional[Union[State, str]] = None,
        showZero: Union[State, bool] = False,
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        variant: Union[State, str] = "standard",
        text: Optional[Union[str, State, Callable]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        # Set properties with validation
        self._set_anchorOrigin(anchorOrigin)
        self._set_badgeContent(badgeContent)
        self._set_children(children)
        self._set_classes(classes)
        self._set_color(color)
        self._set_component(component)
        self._set_components(components)
        self._set_componentsProps(componentsProps)
        self._set_invisible(invisible)
        self._set_icon(icon)
        self._set_max(max)
        self._set_overlap(overlap)
        self._set_key(key)
        self._set_showZero(showZero)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)
        self._set_variant(variant)
        self._set_text(text)

        self._size = 32

        self._init_ui()
        self._set_stylesheet()

        i18n.langChanged.connect(self.retranslateUi)
        self.retranslateUi()

        self.theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self.destroyed.connect(self._on_destroyed)

    # Setter and Getter methods for all parameters
    @_validate_param(file_path="qtmui.material.badge", param_name="anchorOrigin", supported_signatures=Union[State, Dict, type(None)])
    def _set_anchorOrigin(self, value):
        if value is None:
            value = {"vertical": "top", "horizontal": "right"}
        if isinstance(value, dict):
            if value.get("vertical") not in self.VALID_ANCHOR_VERTICAL:
                raise ValueError(f"anchorOrigin.vertical must be one of {self.VALID_ANCHOR_VERTICAL}")
            if value.get("horizontal") not in self.VALID_ANCHOR_HORIZONTAL:
                raise ValueError(f"anchorOrigin.horizontal must be one of {self.VALID_ANCHOR_HORIZONTAL}")
        self._anchorOrigin = value

    def _get_anchorOrigin(self):
        return self._anchorOrigin.value if isinstance(self._anchorOrigin, State) else self._anchorOrigin

    # @_validate_param(file_path="qtmui.material.badge", param_name="badgeContent", supported_signatures=Union[State, Any, type(None)])
    def _set_badgeContent(self, value):
        self._badgeContent = value

    def _get_badgeContent(self):
        return self._badgeContent.value if isinstance(self._badgeContent, State) else self._badgeContent

    # @_validate_param(file_path="qtmui.material.badge", param_name="children", supported_signatures=Union[State, Any, type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.badge", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.badge", param_name="color", supported_signatures=Union[State, str], valid_values=VALID_COLORS)
    def _set_color(self, value):
        self._color = value

    def _get_color(self):
        return self._color.value if isinstance(self._color, State) else self._color

    # @_validate_param(file_path="qtmui.material.badge", param_name="component", supported_signatures=Union[State, Any, type(None)])
    def _set_component(self, value):
        self._component = value

    def _get_component(self):
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.badge", param_name="components", supported_signatures=Union[State, Dict, type(None)])
    def _set_components(self, value):
        self._components = value or {}

    def _get_components(self):
        return self._components.value if isinstance(self._components, State) else self._components

    @_validate_param(file_path="qtmui.material.badge", param_name="componentsProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_componentsProps(self, value):
        self._componentsProps = value or {}

    def _get_componentsProps(self):
        return self._componentsProps.value if isinstance(self._componentsProps, State) else self._componentsProps

    @_validate_param(file_path="qtmui.material.badge", param_name="invisible", supported_signatures=Union[State, bool])
    def _set_invisible(self, value):
        self._invisible = value

    def _get_invisible(self):
        return self._invisible.value if isinstance(self._invisible, State) else self._invisible

    @_validate_param(file_path="qtmui.material.badge", param_name="icon", supported_signatures=Union[State, str, type(None)])
    def _set_icon(self, value):
        self._icon = value

    def _get_icon(self):
        return self._icon.value if isinstance(self._icon, State) else self._icon

    @_validate_param(file_path="qtmui.material.badge", param_name="max", supported_signatures=Union[State, int])
    def _set_max(self, value):
        self._max = value

    def _get_max(self):
        return self._max.value if isinstance(self._max, State) else self._max

    @_validate_param(file_path="qtmui.material.badge", param_name="overlap", supported_signatures=Union[State, str], valid_values=VALID_OVERLAP)
    def _set_overlap(self, value):
        self._overlap = value

    def _get_overlap(self):
        return self._overlap.value if isinstance(self._overlap, State) else self._overlap

    @_validate_param(file_path="qtmui.material.badge", param_name="key", supported_signatures=Union[State, str, type(None)])
    def _set_key(self, value):
        self._key = value

    def _get_key(self):
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(file_path="qtmui.material.badge", param_name="showZero", supported_signatures=Union[State, bool])
    def _set_showZero(self, value):
        self._showZero = value

    def _get_showZero(self):
        return self._showZero.value if isinstance(self._showZero, State) else self._showZero

    @_validate_param(file_path="qtmui.material.badge", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value or {}

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.badge", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value or {}

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.badge", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    # @_validate_param(file_path="qtmui.material.badge", param_name="variant", supported_signatures=Union[State, str], valid_values=VALID_VARIANTS)
    def _set_variant(self, value):
        self._variant = value

    def _get_variant(self):
        return self._variant.value if isinstance(self._variant, State) else self._variant

    @_validate_param(file_path="qtmui.material.badge", param_name="text", supported_signatures=Union[State, str, type(None)])
    def _set_text(self, value):
        self._text = value

    def _get_text(self):
        return self._text.value if isinstance(self._text, State) else self._text

    def _init_ui(self):
        self.setObjectName(str(uuid.uuid4()))
        if self._children:
            self.setFixedSize(QSize(self._size, self._size))
        else:
            if self._badgeContent and len(self._badgeContent) > 1:
                self.setFixedHeight(self._size)
            elif self._text:
                self.setFixedHeight(self._size)
            else:
                self.setFixedSize(QSize(self._size, self._size))


    def retranslateUi(self):
        if self._text:
            if isinstance(self._text, Callable):
                self.setText(translate(self._text))
            else:
                self.setText(self._text)


    def _get_variant_icon(self, variant):
        icons = {
            "online": QTMUI_ASSETS.ICONS.ONLINE,
            "alway": QTMUI_ASSETS.ICONS.ALWAY,
            "busy": QTMUI_ASSETS.ICONS.BUSY,
            "offline": QTMUI_ASSETS.ICONS.OFFLINE,
        }
        return icons[variant]
    
    def _get_variant_color(self, variant):
        icons = {
            "online": "#05c46b",
            "alway": "#00d8d6",
            "busy": "#ff5e57",
            "offline": "#d2dae2"
        }
        return icons[variant]


    def _set_stylesheet(self, component_styled=None):
        self.theme = useTheme()

        if not component_styled:
            component_styled = self.theme.components

        MuiFormControlLabel = component_styled[f"MuiFormControlLabel"].get("styles")
        MuiFormControlLabelRootStyle = get_qss_style(MuiFormControlLabel["label"])

        ownerState = {
            "size": self._size
        }

        PyBadge = component_styled[f"PyBadge"].get("styles")
        PyBadge_root = PyBadge["root"](ownerState)[self._color]
        PyBadge_root_qss = get_qss_style(PyBadge_root)
        PyBadge_root_prop_has_icon_qss = get_qss_style(PyBadge_root["props"]["hasIcon"])
        PyBadge_root_prop_has_text_qss = get_qss_style(PyBadge_root["props"]["hasText"])

        self._text_color = PyBadge_root.get("color")
        self._background_color = PyBadge_root.get("background-color")

        if self._variant in ["alway", "online", "busy", "offline"]:
            self._status_variant_color = PyBadge_root["props"][f"{self._variant}Variant"]["color"]

        if self._icon:
            self.setProperty("hasIcon", "true")
        if self._text:
            self.setProperty("hasText", "true")

        if self._variant == "online":
            self.setProperty("variant", "offline")
        elif self._variant == "busy":
            self.setProperty("variant", "busy")
        elif self._variant == "offline":
            self.setProperty("variant", "offline")
        elif self._variant == "alway":
            self.setProperty("variant", "alway")
        else: # invisible
            self.setProperty("variant", "invisible")

        if self._text == "Typography":
            print('self._text_color________', self._text_color)

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        stylesheet = f"""
            #{self.objectName()}[hasIcon=true] {{
                {PyBadge_root_prop_has_icon_qss}
            }}
            #{self.objectName()}[hasText=true] {{
                color: {self._text_color};
                {PyBadge_root_prop_has_text_qss}
            }}
            #{self.objectName()}[variant=online] {{
            }}
            #{self.objectName()}[variant=busy] {{
            }}
            #{self.objectName()}[variant=offline] {{
            }}
            #{self.objectName()}[variant=alway] {{
            }}
            #{self.objectName()}[variant=invisible] {{
            }}

            {sx_qss}

        """
        self.setStyleSheet(stylesheet)
        self.update()

    def paintEvent(self, event):
        if not hasattr(self, "theme") or not hasattr(self, "_background_color"):
            self._set_stylesheet()

        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)

        if self._variant:
            if self._variant in ["alway", "online", "busy", "offline"]:
                painter.setPen(Qt.NoPen)
                painter.setBrush(QBrush(QColor(Qt.gray)))
                painter.drawRoundedRect(0,0,self._size,self._size,self._size/2,self._size/2)

                # Thay đổi màu fill của SVG thành màu mong muốn
                # updated_svg_content = change_svg_color(svg_content, self._text_color)
                updated_svg_content = writeSvg(self._get_variant_icon(self._variant), fill=self._status_variant_color)
                # Tạo QByteArray từ nội dung SVG đã thay đổi
                svg_data = QByteArray(updated_svg_content.encode("utf-8"))
                # Sử dụng QSvgRenderer để render SVG với màu mới
                svg_renderer = QSvgRenderer(svg_data)
                svg_renderer.render(painter, QRect(self._size*3.2/5,self._size*3.2/5,self._size*2/5,self._size*2/5))  # Vẽ SVG vào button
                # painter.drawRoundedRect(self._size*2/4,self._size*2/4,self._size/2,self._size/2,self._size/4,self._size/4)
        if self._children:
            # Render button khác thành một QPixmap
            pixmap = QPixmap(QSize(self._size, self._size))  # Tạo QPixmap có kích thước như button khác
            self._children.render(pixmap)  # Render button khác vào QPixmap
            # Vẽ QPixmap của button khác lên button hiện tại
            # painter.drawPixmap(QRect(0, 0, pixmap.width(), pixmap.height()), pixmap)
            painter.setPen(Qt.NoPen)
            painter.setBrush(QBrush(QColor(Qt.darkYellow)))
            if self._overlap == "circular":
                painter.drawRoundedRect(0,0,self._size,self._size,self._size/2,self._size/2)
            else:
                painter.drawRect(QRect(self._size/8, self._size/8, pixmap.width()-self._size/4, pixmap.height()-self._size/4))

            painter.setPen(Qt.NoPen)
            painter.setBrush(QBrush(QColor(self._background_color)))
            painter.drawRoundedRect(self._size*3/4,0,self._size/4,self._size/4,self._size/8,self._size/8)
        else:
            badge_w = 0
            if self._badgeContent and len(self._badgeContent) > 1:
                badge_w = len(self._badgeContent) * 5
            self.setMinimumWidth(self._size + badge_w)

            badge_pos_x_text_mode = 0
            if self._text and len(self._text) > 1:
                badge_pos_x_text_mode = len(self._text) * 6

                self.setMinimumWidth(self._size + badge_pos_x_text_mode)

            if self._icon:
                pixmap = QPixmap(self._icon)
                scaled_pixmap = pixmap.scaled(self.size(), Qt.AspectRatioMode.KeepAspectRatioByExpanding, Qt.TransformationMode.SmoothTransformation)
                # painter.setPen(Qt.NoPen)
                # brush = QBrush(QColor("#000000"))
                # brush.setStyle(Qt.Dense2Pattern)
                # brush.setTexture(scaled_pixmap)
                # painter.setBrush(brush)
                # painter.drawRoundedRect(self.rect(), self._size, self._size)

                # draw giữ nguyên màu sgv
                # painter.drawPixmap(self.rect(), scaled_pixmap)



                # Thay đổi màu fill của SVG thành màu mong muốn
                # updated_svg_content = change_svg_color(svg_content, self._text_color)
                updated_svg_content = writeSvg(self._icon, fill=self.theme.palette.text.secondary)
                # Tạo QByteArray từ nội dung SVG đã thay đổi
                svg_data = QByteArray(updated_svg_content.encode("utf-8"))
                # Sử dụng QSvgRenderer để render SVG với màu mới
                svg_renderer = QSvgRenderer(svg_data)
                svg_renderer.render(painter, self.rect().adjusted(5,5,-badge_w-5,-5))  # Vẽ SVG vào button

            if self._variant == "dot":
                painter.setPen(Qt.NoPen)
                painter.setBrush(QBrush(QColor(self._background_color)))
                painter.drawRoundedRect(self._size*3/4+badge_pos_x_text_mode,0,self._size/4,self._size/4,self._size/8,self._size/8)

            elif self._badgeContent:
                painter.setPen(Qt.NoPen)
                brush = QBrush(QColor(self._background_color))
                painter.setBrush(brush)
                painter.drawRoundedRect(self._size/2,0,self._size/2+badge_w,self._size/2,self._size/4,self._size/4)

                # Ghi đè font-size của QPushButton cha
                custom_font = QFont(self.font())  # Lấy font của QPushButton cha
                custom_font.setPointSize(10)  # Thay đổi kích thước font, ví dụ: 12px
                painter.setFont(custom_font)  # Áp dụng font mới cho painter

                painter.setBrush(Qt.NoBrush)
                painter.setPen(QPen(Qt.white, 1))
                painter.drawText(QRect(self._size*3/4 - self._size/8,0,self._size/2+badge_w,self._size/2), Qt.AlignVCenter, self._badgeContent)

        super().paintEvent(event)



import uuid
from PySide6.QtWidgets import QWidget, QFrame, QHBoxLayout, QSizePolicy
from PySide6.QtCore import Qt
from typing import Union, List, Callable

from ..box import Box
from ..spacer import HSpacer
from .timeline_content import TimelineContent, AlignBox
from .timeline_opposite_content import TimelineOppositeContent

class TimelineItem(QFrame):
    def __init__(self, 
                 children=None,  # Nội dung của component (node)
                 classes: dict = None,  # Ghi đè hoặc mở rộng styles cho component (dict)
                 position: str = 'right',  # Vị trí của item trên timeline ('alternate-reverse' | 'alternate' | 'left' | 'right')
                 key: Union[str, int] = None,  # Vị trí của item trên timeline ('alternate-reverse' | 'alternate' | 'left' | 'right')
                 sx: Union[List[Union[Callable, dict, bool]], Callable, dict] = None  # Thuộc tính hệ thống, thêm CSS bổ sung (array | func | object)
                 ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        # self.setStyleSheet(f"#{self.objectName()} {{background-color: blue;}}")
        
        self._children = children
        self._classes = classes if classes is not None else {}
        self._position = position
        self._key = key
        self._sx = sx

        # Thiết lập layout cơ bản cho item
        self._initUI()

    def _initUI(self):
        """Khởi tạo giao diện của TimelineItem."""
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        
        self._add_children()
        self._setup_layout()

    # @is_list
    def _add_children(self):
        # Thêm các children (nếu có)
        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            if self._position == "alternate":
                self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
                if self._key % 2 == 1:
                    spacer_w = 0
                    spacer_w_oppo = 0
                    for child in reversed(self._children):
                        if child:
                            if isinstance(child, TimelineOppositeContent):
                                spacer_w_oppo = child.sizeHint().width()
                            if isinstance(child, TimelineContent):
                                spacer_w = child.sizeHint().width()
                            self.layout().addWidget(child)

                    
                    if not len(self.findChildren(TimelineOppositeContent)):
                        self.layout().addWidget(HSpacer(width=spacer_w))
                    else:
                        max_size_content = spacer_w if (spacer_w - spacer_w_oppo) > 0 else spacer_w_oppo
                        max_size_content = max_size_content + 12
                        for index, child in enumerate(self.findChildren(QWidget), 0):
                            if index == 0:
                                if isinstance(child.content_widget, Box):
                                    child.content_widget.layout().setAlignment(Qt.AlignRight)
                                    for alignBox in child.content_widget.findChildren(AlignBox):
                                        alignBox.layout().insertWidget(0, HSpacer())
                                else:
                                    child.content_widget.setAlignment(Qt.AlignRight)

                            if isinstance(child, TimelineOppositeContent):
                                child.setFixedWidth(max_size_content)
                            if isinstance(child, TimelineContent):
                                child.setFixedWidth(max_size_content)

                else:
                    spacer_w = 0
                    spacer_w_oppo = 0
                    for child in self._children:
                        if child:
                            if isinstance(child, TimelineOppositeContent):
                                spacer_w_oppo = child.sizeHint().width()
                            if isinstance(child, TimelineContent):
                                spacer_w = child.sizeHint().width()
                            self.layout().addWidget(child)
                    if not len(self.findChildren(TimelineOppositeContent)):
                        self.layout().insertWidget(0, HSpacer(width=spacer_w))
                    else:
                        max_size_content = spacer_w if (spacer_w - spacer_w_oppo) > 0 else spacer_w_oppo
                        max_size_content = max_size_content + 12
                        for index, child in enumerate(self.findChildren(QWidget), 0):
                            if index == 0:
                                if isinstance(child.content_widget, Box):
                                    child.content_widget.layout().setAlignment(Qt.AlignRight)
                                    for alignBox in child.content_widget.findChildren(AlignBox):
                                        alignBox.layout().insertWidget(0, HSpacer())
                                else:
                                    child.content_widget.setAlignment(Qt.AlignRight)

                            if isinstance(child, TimelineOppositeContent):
                                child.setFixedWidth(max_size_content)
                            if isinstance(child, TimelineContent):
                                child.setFixedWidth(max_size_content)
            else:
                for child in reversed(self._children) if self._position == 'left' else self._children:
                    if child:
                        self.layout().addWidget(child)

    def _setup_layout(self):

        # Thiết lập vị trí cho timeline item dựa trên giá trị position
        if self._position == 'left':
            self.setAlignment(Qt.AlignLeft)
        elif self._position == 'right':
            self.setAlignment(Qt.AlignRight)
        elif self._position == 'alternate':
            self.setAlignment(Qt.AlignCenter)  # Giả định rằng alternate căn giữa
        elif self._position == 'alternate-reverse':
            self.setAlignment(Qt.AlignJustify)  # Hoặc xử lý khác tùy thuộc vào logic

    def setAlignment(self, alignment):
        """Phương thức để căn chỉnh item dựa trên vị trí."""
        self.layout().setAlignment(alignment)


from PySide6.QtWidgets import QSpacerItem, QWidget, QVBoxLayout, QSizePolicy

class VSpacer(QWidget):
    def __init__(self, height: int = None):
        super().__init__()
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        if height is not None:
            self.layout().addItem(QSpacerItem(0, height, QSizePolicy.Minimum, QSizePolicy.Fixed))
        else:
            self.layout().addItem(QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding))


from math import cos, pi

from PySide6.QtWidgets import QGraphicsEffect
from PySide6.QtCore import Property, QPointF, QPoint, Qt, QRectF
from PySide6.QtGui import QPainter, QPixmap, QLinearGradient, QColor, QBrush, QGradient

from qtmui.material.styles import useTheme

class AnimEffect(QGraphicsEffect):
    """Flip theo trục X, mô phỏng bằng scaleY = cos(angle)."""

    PRESET_COLORS = {
        "ocean": {
            "light": ["#2193b0", "#6dd5ed"],
            "dark": ["#141e30", "#243b55"],
        },
        "sunset": {
            "light": ["#ff512f", "#f09819"],
            "dark": ["#3a1c71", "#d76d77"],
        },
        "forest": {
            "light": ["#11998e", "#38ef7d"],
            "dark": ["#0f2027", "#203a43"],
        },
    }


    def __init__(self, parent=None, src: str = None, variants: dict = None):
        super().__init__(parent)
        self._src = src
        
        if src:
            self._pixmap = QPixmap(src)
        
        self._opacity = 1.0  # 0.0 → 1.0
        self._angle = 0.0  # độ, -360 → 0 hoặc 0 → 360
        self._offsetX = 0.0
        self._offsetY = 0.0
        self._rotationX = 0.0  # độ, -180 → 0 hoặc 0 → 180
        self._rotationY = 0.0  # độ, -180 → 0 hoặc 0 → 180
        self._scaleX = 1.0
        self._scaleY = 1.0
        self._scale = 1.0
        self.direction = "right"
        
        self._gradient = variants.get("animate", {}).get("gradient")
        self._bgProgress = 0.0
            


    # ############# background image effect ###########
    # def boundingRectFor(self, sourceRect: QRectF) -> QRectF: # background effect
    #     # Override để ngăn chặn size widget thay đổi do effect vẽ ngoài bound.
    #     # Bằng cách return sourceRect, effect bị clip trong bound gốc, không mở rộng size.
    #     return sourceRect

    def getOffsetX(self):
        return self._offsetX

    def setOffsetX(self, val):
        self._offsetX = val
        self.update()

    offsetX = Property(float, getOffsetX, setOffsetX)
    
    def getOffsetY(self):
        return self._offsetY

    def setOffsetY(self, val):
        self._offsetY = val
        self.update()

    offsetY = Property(float, getOffsetY, setOffsetY)
    
    def getAngle(self):
        return self._angle

    def setAngle(self, val):
        self._angle = val
        self.update()

    angle = Property(float, getAngle, setAngle)
    
    def getRotationX(self):
        return self._rotationX

    def setRotationX(self, val):
        self._rotationX = val
        self.update()

    rotationX = Property(float, getRotationX, setRotationX)
    
    def getRotationY(self):
        return self._rotationY

    def setRotationY(self, val):
        self._rotationY = val
        self.update()

    rotationY = Property(float, getRotationY, setRotationY)
    
    def getScaleX(self):
        return self._scaleX

    def setScaleX(self, val):
        self._scaleX = val
        self.update()

    scaleX = Property(float, getScaleX, setScaleX)
    
    def getScaleY(self):
        return self._scaleY

    def setScaleY(self, val):
        self._scaleY = val
        self.update()

    scaleY = Property(float, getScaleY, setScaleY)
    
    def getOpacity(self):
        return self._opacity

    def setOpacity(self, val):
        self._opacity = val
        self.update()

    opacity = Property(float, getOpacity, setOpacity)


    def getScale(self):
        return self._scale

    def setScale(self, v):
        self._scale = v
        self.update()

    scale = Property(float, getScale, setScale)
    
    def getBgProgress(self):
        return self._bgProgress

    def setBgProgress(self, v):
        self._bgProgress = v
        self.update()

    bgProgress = Property(float, getBgProgress, setBgProgress)
    



    # ---------------- Resolve colors ----------------
    def _resolve_colors(self, colors):
        if isinstance(colors, str):
            preset = self.PRESET_COLORS.get(colors)
            return preset[useTheme().palette.mode.lower()]
        return list(colors)

    # ---------------- Generate colors ----------------
    def _generate_colors(self):
        steps = self._gradient.get("steps")
        interpolate = self._gradient.get("interpolate")
        colors = self._resolve_colors(self._gradient.get("colors"))
        
        if not steps:
            return colors
        
        start = QColor(colors[0])
        end = QColor(colors[1])

        result = []

        for i in range(steps):
            t = i / (steps - 1)

            if interpolate == "hsl":
                c = QColor.fromHsl(
                    int(start.hslHue() + t * (end.hslHue() - start.hslHue())),
                    int(start.hslSaturation() + t * (end.hslSaturation() - start.hslSaturation())),
                    int(start.lightness() + t * (end.lightness() - start.lightness())),
                )
            else:
                c = QColor(
                    int(start.red() + t * (end.red() - start.red())),
                    int(start.green() + t * (end.green() - start.green())),
                    int(start.blue() + t * (end.blue() - start.blue())),
                )

            result.append(c)

        return result


    def _draw_background_gradient(self, painter: QPainter, w: int, h: int):
        """
        Vẽ background gradient + pan animation
        """
        # colors = self._gradient.get("colors", ["#ee7752", "#e73c7e", "#23a6d5", "#23d5ab"])
        colors = self._generate_colors()
        
        direction = self._gradient.get("direction", "right")
        
        grad = None
        offset = 0.0

        colors_ext = colors + [colors[0]]
        num_intervals = len(colors)

        if direction in ("top", "bottom"):
            fill_factor = 6
            fill_h = h * fill_factor
            fill_w = w
            grad = QLinearGradient(0, 0, 0, fill_h)
            for i in range(len(colors_ext)):
                grad.setColorAt(float(i) / num_intervals, QColor(colors_ext[i]))

            pos = self._bgProgress
            if direction == "top":
                pos = 1 - pos  # reverse for top (pan upwards)
            offset = -pos * fill_h
            grad.setStart(0, offset)
            grad.setFinalStop(0, offset + fill_h)

        else:  # left, right
            fill_factor = 6
            fill_w = w * fill_factor
            fill_h = h
            grad = QLinearGradient(0, 0, fill_w, 0)
            for i in range(len(colors_ext)):
                grad.setColorAt(float(i) / num_intervals, QColor(colors_ext[i]))

            pos = self._bgProgress
            if direction == "left":
                pos = 1 - pos  # reverse for left (pan leftwards)
            offset = -pos * fill_w
            grad.setStart(offset, 0)
            grad.setFinalStop(offset + fill_w, 0)

        if grad:
            grad.setSpread(QGradient.RepeatSpread)
            painter.fillRect(0, 0, w, h, grad)
            
            
    def draw(self, painter: QPainter):
        # if hasattr(self, "_pixmap"):
        #     pixmap = self._pixmap
        # else:
        #     pixmap = self.sourcePixmap()
        pixmap = self.sourcePixmap()
            
        if pixmap.isNull():
            print('pixmap.isNull()')
            return

        painter.save()
        painter.setRenderHints(
            QPainter.Antialiasing | QPainter.SmoothPixmapTransform
        )
        
        ################### opacity ###################
        painter.setOpacity(self._opacity)


        ################### Flip Scale X/Y ###################
        w, h = pixmap.width(), pixmap.height()
        cx, cy = w / 2, h / 2

        # Dịch tâm
        painter.translate(cx, cy)
        # Góc radian
        radX = self._rotationX * pi / 180.0
        radY = self._rotationY * pi / 180.0
        scale_x = cos(radX)
        scale_y = cos(radY)
        # Giữ dấu để flip
        if self._rotationX == 0.0 and self._rotationY == 0.0:
            painter.scale(self._scaleX, self._scaleY)
        else:
            painter.scale(scale_y, scale_x)
        
        # Dịch tâm về lại
        painter.translate(-cx, -cy)
        

        ################### Flip Scale X/Y ###################
        if self._angle != 0.0:
            offset = QPoint()
            pixmap = self.sourcePixmap(Qt.DeviceCoordinates)
            
            cx = pixmap.width() / 2
            cy = pixmap.height() / 2

            painter.translate(QPointF(offset) + QPointF(cx, cy))
            painter.rotate(self._angle)
            painter.translate(-cx, -cy)

            painter.drawPixmap(0, 0, pixmap)
        
        
        ################### background gradient ###################
        if self._gradient:
            self._draw_background_gradient(painter, w, h)
        
        
        ################### background ###################
        if self._src:
            progress = self._scale/ 0.1 - 1.0
            pan_x = 0.0
            pan_y = 0.0
            
            pan_amount = 0.1 * h if self.direction in ("top", "bottom") else 0.1 * w

            if self.direction == "top":
                pan_y = - progress * pan_amount  # Dịch image lên (pan to top)
            elif self.direction == "bottom":
                pan_y = progress * pan_amount   # Dịch image xuống
            elif self.direction == "left":
                pan_x = -progress * pan_amount  # Dịch image trái
            elif self.direction == "right":
                pan_x = progress * pan_amount   # Dịch image phải
                
            scaled_pix = self._pixmap.scaled(
                int(w * self._scale), int(h * self._scale),
                Qt.KeepAspectRatioByExpanding,
                Qt.SmoothTransformation
            )

            # Tính vị trí vẽ để center, rồi thêm pan
            draw_x = (w - scaled_pix.width()) / 2.0 + pan_x
            draw_y = (h - scaled_pix.height()) / 2.0 + pan_y
            
            # Vẽ scaled pixmap tại vị trí điều chỉnh
            # painter.drawPixmap(draw_x, 0, scaled_pix)
            painter.drawPixmap(0, 0, scaled_pix)
            
        
        # Vẽ pixmap
        if not self._angle:
            self.drawSource(painter)
            
        painter.restore()



import asyncio
import uuid
from typing import Optional, List, Callable, Any
from PySide6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QButtonGroup, QFrame, QSizePolicy
from PySide6.QtCore import Qt, Signal, QTimer
from PySide6.QtGui import QColor

from qtmui.hooks import State

from .toggle_button import ToggleButton

class ToggleButtonGroup(QFrame):
    valueChanged = Signal(object)  # Tín hiệu được phát khi giá trị thay đổi

    def __init__(
        self,
        children: List[ToggleButton],  # Thay đổi từ buttons thành children
        exclusive: bool = False,
        key: Optional[str] = None,
        color: str = 'default',
        size: str = 'medium',
        orientation: str = 'horizontal',
        fullWidth: bool = False,
        disabled: bool = False,
        value: Optional[Any] = None,
        onChange: Optional[Callable[[Any], None]] = None,
        parent: Optional[QWidget] = None
    ):
        super().__init__(parent)
        self.setObjectName(str(uuid.uuid4()))

        self._children = children  # Đổi từ _buttons thành _children
        self._exclusive = exclusive
        self._color = color
        self._size = size
        self._orientation = orientation
        self._fullWidth = fullWidth
        self._disabled = disabled
        self._state = value

        if isinstance(self._state, State):
            self._value = self._state.value
            self._state.valueChanged.connect(self._on_state_changed)
            self._set_state(self._state.value)

        self._onChange = onChange

        # Tạo ButtonGroup cho chức năng chọn độc quyền (exclusive)
        self._button_group = QButtonGroup(self)
        if self._exclusive:
            self._button_group.setExclusive(True)

        # Thiết lập layout dựa trên orientation
        if self._orientation == 'horizontal':
            self.setLayout(QHBoxLayout())
        else:
            self.setLayout(QVBoxLayout())

        self.layout().setContentsMargins(4,4,4,4)
        self.layout().setSpacing(5)
        self.layout().setAlignment(Qt.AlignCenter)

        # Thêm các nút vào nhóm và thiết lập sự kiện
        for button in self._children:  # Thay đổi thành _children
            self._add_button(button)

        border_color = "rgba(145, 158, 171, 0.1)"
        self.setStyleSheet(
            f"""
                #{self.objectName()} {{
                    background-color: white;
                    border: 1px solid {border_color};
                    border-radius: 8px;
                }}

            """
        )

    def _on_state_changed(self, newValue):
        self._value = newValue
        self.valueChanged.emit(self._value)
        self._set_state(newValue)

    def _add_button(self, button: ToggleButton):
        """Thêm một nút vào ToggleButtonGroup."""
        if self._fullWidth:
            button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        else:
            button.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)

        button.clicked.connect(self._on_button_clicked)
        if self._color:
            button._color = self._color
        if self._size:
            button._size = self._size

        if self._disabled:
            self.setEnabled(not self._disabled)

        QTimer.singleShot(0, button._scheduleSetStyleSheet)

        self.layout().addWidget(button)

    def _on_button_clicked(self):
        """Hàm xử lý khi một nút được click."""
        clicked_button: ToggleButton = self.sender()
        # Phát tín hiệu thay đổi giá trị
        if self._exclusive:
            if self._onChange:
                self._onChange(clicked_button._key)
        else:
            if self._onChange:
                if clicked_button._key in self._value:
                    self._value.remove(clicked_button._key)
                else:
                    self._value.append(clicked_button._key)
                self._onChange(self._value)

    def _set_state(self, value: Any):
        """Đặt giá trị hiện tại cho ToggleButtonGroup."""
        if self._exclusive:
            # Chỉ chọn một nút
            for button in self._children:
                button.set_selected(button._key == value)
        else:
            # Có thể chọn nhiều nút
            for button in self._children:
                button.set_selected(button._key in value)

    def get_value(self):
        """Trả về giá trị hiện tại của ToggleButtonGroup."""
        return self._state

    def set_disabled(self, disabled: bool):
        """Vô hiệu hóa tất cả các nút trong ToggleButtonGroup."""
        self._disabled = disabled
        for button in self._children:
            button.setDisabled(disabled)





import threading, time
from PySide6.QtWidgets import QVBoxLayout, QWidget, QSizePolicy, QFrame
from PySide6.QtGui import QPainter
from PySide6.QtCore import Qt, QPoint, QRect, Signal, QTimer
from .view import View
from qtmui.hooks import useState

class Page(QFrame):
    renderView = Signal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)


    def _wait_for_3s(self):
        time.sleep(2)
        self.renderView.emit()


    def add_widget(self, element: QWidget):
        """Thêm một phần tử giao diện vào trang."""
        if isinstance(element, QWidget):
            self.layout().addWidget(element)
        else:
            print(f"Opp!! element must have type QWidget")
        
  


import asyncio
import json
import uuid
from typing import Optional, Union, Dict, Callable

from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QFrame, QWidget, QApplication, QSizePolicy
from PySide6.QtCore import Qt, QEvent
from qtmui.hooks import State
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.styles import useTheme
from ..widget_base import PyWidgetBase

from ..utils.validate_params import _validate_param

class Container(QFrame, PyWidgetBase):
    """
    A component that centers and constrains the width of its content based on breakpoints.
    """

    # Valid values for maxWidth
    VALID_MAX_WIDTH = ["xs", "sm", "md", "lg", "xl", False]

    # Define max-width based on breakpoints (from MUI)
    MAX_WIDTHS = {
        "xs": 0,    # No limit
        "sm": 600,
        "md": 900,
        "lg": 1200,
        "xl": 1536,
    }

    # Define min-width based on breakpoints (used when fixed=True)
    MIN_WIDTHS = {
        "xs": 0,
        "sm": 600,
        "md": 900,
        "lg": 1200,
        "xl": 1536,
    }

    def __init__(
        self,
        key: str = None,
        disableGutters: Optional[Union[bool, State]] = False,
        fixed: Optional[Union[bool, State]] = False,
        maxWidth: Optional[Union[str, bool, State]] = "lg",
        sx: Optional[Union[State, Callable, str, Dict]] = None,
        children: Optional[Union[State, list]] = None,
        **kwargs
    ):
        # Initialize parent QFrame
        super().__init__(**kwargs)
        self.setObjectName(str(uuid.uuid4()))
        PyWidgetBase._setUpUi(self)
        
        self._key = key
        self.theme = useTheme()
        self._current_breakpoint = None

        # List to hold references to child widgets to prevent Qt from deleting them
        self._widget_references = []

        # Assign values to properties
        self._set_disableGutters(disableGutters)
        self._set_fixed(fixed)
        self._set_maxWidth(maxWidth)
        self._set_children(children)
        self._set_sx(sx)

        # from PyWidgetBase
        self._setup_sx_position(sx)

        # Initialize UI
        self._init_ui()

    # ...existing validation and getter methods...
    @_validate_param(file_path="qtmui.material.container", param_name="disableGutters", supported_signatures=Union[bool, State])
    def _set_disableGutters(self, value):
        self._disableGutters = value

    def _get_disableGutters(self):
        return self._disableGutters.value if isinstance(self._disableGutters, State) else self._disableGutters

    @_validate_param(file_path="qtmui.material.container", param_name="fixed", supported_signatures=Union[bool, State])
    def _set_fixed(self, value):
        self._fixed = value

    def _get_fixed(self):
        return self._fixed.value if isinstance(self._fixed, State) else self._fixed

    @_validate_param(file_path="qtmui.material.container", param_name="maxWidth", supported_signatures=Union[str, bool, State], valid_values=VALID_MAX_WIDTH)
    def _set_maxWidth(self, value):
        self._maxWidth = value

    def _get_maxWidth(self):
        return self._maxWidth.value if isinstance(self._maxWidth, State) else self._maxWidth

    @_validate_param(file_path="qtmui.material.container", param_name="children", supported_signatures=Union[State, list, type(None)])
    def _set_children(self, value):
        self._widget_references.clear()
        self._children = value
        children = self._get_children()

        if isinstance(children, list):
            for child in children:
                if child is None:
                    continue
                if isinstance(child, QWidget):
                    self._widget_references.append(child)
                elif isinstance(child, list):
                    for sub_child in child:
                        if isinstance(sub_child, QWidget):
                            self._widget_references.append(sub_child)
                else:
                    raise TypeError(f"Each element in children must be a QWidget, but got {type(child)}")
        elif children is not None:
            raise TypeError(f"children must be a State or list, but got {type(children)}")

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.container", param_name="sx", supported_signatures=Union[State, Callable, str, Dict, type(None)])
    def _set_sx(self, value):
        self._sx = value

    # ...existing signal methods...
    def _connect_signals(self):
        if isinstance(self._disableGutters, State):
            self._disableGutters.valueChanged.connect(self._on_disableGutters_changed)
        if isinstance(self._fixed, State):
            self._fixed.valueChanged.connect(self._on_fixed_changed)
        if isinstance(self._maxWidth, State):
            self._maxWidth.valueChanged.connect(self._on_maxWidth_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.connect(self._on_sx_changed)
        if isinstance(self._children, State):
            self._children.valueChanged.connect(self._on_children_changed)

    def _on_disableGutters_changed(self):
        self._set_disableGutters(self._disableGutters)
        self._update_margins()

    def _on_fixed_changed(self):
        self._set_fixed(self._fixed)
        self.adjustWidth()

    def _on_maxWidth_changed(self):
        self._set_maxWidth(self._maxWidth)
        self.adjustWidth()

    def _on_sx_changed(self):
        self._set_sx(self._sx)
        self._set_stylesheet()

    def _on_children_changed(self):
        self._set_children(self._children)
        self._clear_layout()
        self._add_children_to_layout()

    def _init_ui(self):
        """Initialize the Container UI with QHBoxLayout and widgetContents."""
        # Use QHBoxLayout for Container to center the content widget
        self.setLayout(QHBoxLayout())
        self.layout().setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.layout().setContentsMargins(0, 0, 0, 0)

        # Create a content widget to hold children - this will have responsive width
        self.widgetContents = QFrame()
        self.widgetContents.setObjectName(str(uuid.uuid4()))
        self.contentLayout = QVBoxLayout(self.widgetContents)
        self.contentLayout.setAlignment(Qt.AlignmentFlag.AlignTop)
        self.contentLayout.setContentsMargins(0, 0, 0, 0)

        # Add the content widget to the main layout - this auto-centers it
        self.layout().addWidget(self.widgetContents)

        # Update margins based on disableGutters
        self._update_margins()

        # Add child components
        self._add_children_to_layout()

        # Update positions of child components with position: absolute
        self.update_absolute_children()

        self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()
        self.destroyed.connect(self._on_destroyed)

        # Connect signals for State parameters
        self._connect_signals()

    def _add_children_to_layout(self):
        """Add child components to the contentLayout."""
        if self._children is not None:
            children = self._get_children()
            if isinstance(children, list):
                for child in children:
                    if child is None:
                        continue
                    if isinstance(child, QWidget):
                        # Check if the widget has position: absolute
                        sx = getattr(child, '_sx', None)
                        if sx and isinstance(sx, dict) and sx.get("position") == "absolute":
                            self._child_abs.append(child)
                            child._relative_to = self
                            self.contentLayout.addWidget(child)
                        else:
                            self.contentLayout.addWidget(child)
                    else:
                        raise TypeError(f"Each element in children must be a QWidget, but got {type(child)}")

    def _update_margins(self):
        """Update margins based on disableGutters."""
        if hasattr(self, 'contentLayout'):
            disable_gutters = self._get_disableGutters()
            if disable_gutters:
                self.contentLayout.setContentsMargins(0, 0, 0, 0)
            else:
                self.contentLayout.setContentsMargins(16, 0, 16, 0)  # Default padding of 16px on left/right

    def _clear_layout(self):
        """Remove all widgets from the contentLayout."""
        if hasattr(self, 'contentLayout'):
            while self.contentLayout.count():
                item = self.contentLayout.takeAt(0)
                if item.widget():
                    widget = item.widget()
                    widget.setParent(None)
                    if hasattr(self, '_child_abs') and widget in self._child_abs:
                        self._child_abs.remove(widget)
                        widget._relative_to = None

    def adjustWidth(self):
        """Adjust the widgetContents width based on parent size, fixed, and maxWidth settings."""
        if not hasattr(self, 'widgetContents'):
            return
            
        if not self.parent():
            return
            
        parent_width = self.parent().width()
        
        # Get current settings
        fixed = self._get_fixed()
        max_width_setting = self._get_maxWidth()
        
        # Determine current MUI breakpoint based on parent width
        if parent_width < 600:  # xs
            current_breakpoint = "xs"
        elif parent_width < 900:  # sm
            current_breakpoint = "sm"
        elif parent_width < 1200:  # md
            current_breakpoint = "md"
        elif parent_width < 1536:  # lg
            current_breakpoint = "lg"
        else:  # xl
            current_breakpoint = "xl"
        
        # Default behavior: widgetContents equals parent width
        new_width = parent_width
        
        # Apply maxWidth constraint first
        if max_width_setting != False and max_width_setting in self.MAX_WIDTHS:
            max_width_limit = self.MAX_WIDTHS[max_width_setting]
            if max_width_limit > 0:  # 0 means no limit (xs case)
                new_width = min(new_width, max_width_limit)
        
        # Apply fixed constraint - use width of previous breakpoint
        if fixed:
            if current_breakpoint == "xl":
                fixed_width = self.MIN_WIDTHS["lg"]  # 1200
            elif current_breakpoint == "lg":
                fixed_width = self.MIN_WIDTHS["md"]  # 900
            elif current_breakpoint == "md":
                fixed_width = self.MIN_WIDTHS["sm"]  # 600
            elif current_breakpoint == "sm":
                fixed_width = self.MIN_WIDTHS["xs"]  # 0 (full width)
            else:  # xs
                fixed_width = 0  # Full width for xs
                
            if fixed_width > 0:
                new_width = fixed_width
        
        # Final constraint: ensure not wider than parent
        new_width = min(new_width, parent_width)
        
        # Set the width on widgetContents - this automatically centers due to QHBoxLayout
        self.widgetContents.setMaximumWidth(int(new_width))
        
        print(f"Container width: parent={parent_width}, breakpoint={current_breakpoint}, fixed={fixed}, maxWidth={max_width_setting}, final_width={new_width}")

    def eventFilter(self, obj, event):
        """Monitor events of the parent to track size changes."""
        if obj == self.parent() and event.type() == QEvent.Resize:
            self.adjustWidth()
        return super().eventFilter(obj, event)

    def _on_destroyed(self):
        """Disconnect signals when the object is destroyed."""
        if isinstance(self._disableGutters, State):
            self._disableGutters.valueChanged.disconnect(self._on_disableGutters_changed)
        if isinstance(self._fixed, State):
            self._fixed.valueChanged.disconnect(self._on_fixed_changed)
        if isinstance(self._maxWidth, State):
            self._maxWidth.valueChanged.disconnect(self._on_maxWidth_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.disconnect(self._on_sx_changed)
        if isinstance(self._children, State):
            self._children.valueChanged.disconnect(self._on_children_changed)

    def _set_stylesheet(self, component_styled=None):
        """Set the stylesheet for the Container."""
        self.theme = useTheme()

        ownerState = {}
        if not component_styled:
            component_styled = self.theme.components
        PyContainer_root = component_styled["PyContainer"].get("styles")["root"]
        PyContainer_root_qss = get_qss_style(PyContainer_root)

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, State):
                sx = self._sx.value
            elif isinstance(self._sx, Callable):
                sx = self._sx()
            else:
                sx = self._sx

            if isinstance(sx, dict):
                sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
            elif isinstance(sx, str) and sx != "":
                sx_qss = sx

        # Apply classes corresponding to MUI
        classes = ["MuiContainer-root"]
        disable_gutters = self._get_disableGutters()
        fixed = self._get_fixed()
        max_width = self._get_maxWidth()

        if disable_gutters:
            classes.append("MuiContainer-disableGutters")
        if fixed:
            classes.append("MuiContainer-fixed")
        if max_width in self.VALID_MAX_WIDTH:
            classes.append(f"MuiContainer-maxWidth{max_width.capitalize()}")

        class_styles = " ".join(classes)

        stylesheet = f"""
            #{self.objectName()} {{
                {PyContainer_root_qss}
            }}
            {sx_qss}
        """
        self.setStyleSheet(stylesheet)

    def showEvent(self, e):
        """ fade in """
        self.adjustWidth()
        PyWidgetBase.showEvent(self)
        super().showEvent(e)

    def paintEvent(self, arg__1):
        """Handle paint events."""
        PyWidgetBase.paintEvent(self, arg__1)
        return super().paintEvent(arg__1)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.adjustWidth()
        PyWidgetBase.resizeEvent(self, event)


import uuid
import threading
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QMenu, QFrame, QVBoxLayout, QWidget, QScrollArea
from PySide6.QtCore import Qt, QPoint, Signal
from PySide6.QtGui import QFocusEvent, QHideEvent
from qtmui.hooks import State, useEffect
from qtmui.material.styles import useTheme
from ..utils.validate_params import _validate_param
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from .menu_scroll_area import MenuScrollArea

class Menu(QMenu):
    """
    A component that renders a menu, styled like Material-UI Menu, with support for positioning and transitions.

    The `Menu` component displays a list of menu items, typically `MenuItem` components, with support for
    positioning relative to an anchor element, focus management, and styling overrides.

    Parameters
    ----------
    open : State or bool
        If True, the menu is shown (required).
        Can be a `State` object for dynamic updates.
    anchorEl : State, QWidget, Callable, or None, optional
        An element or function that returns one, used to set the position of the menu. Default is None.
        Can be a `State` object for dynamic updates.
    autoFocus : State or bool, optional
        If True, focuses the menu or first item on open. Default is True.
        Can be a `State` object for dynamic updates.
    children : State, List[QWidget], QWidget, or None, optional
        Menu contents, normally MenuItems. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    disableAutoFocusItem : State or bool, optional
        If True, prevents focusing the active item on open. Default is False.
        Can be a `State` object for dynamic updates.
    id : State or str, optional
        The identifier for the component. Default is None.
        Can be a `State` object for dynamic updates.
    MenuListProps : State or dict, optional
        Props applied to the MenuList element. Default is None.
        Deprecated: Use `slotProps.list` instead.
        Can be a `State` object for dynamic updates.
    minWidth : State or int, optional
        The minimum width of the menu in pixels. Default is None.
        Can be a `State` object for dynamic updates.
    maxWidth : State or int, optional
        The maximum width of the menu in pixels. Default is None.
        Can be a `State` object for dynamic updates.
    maxHeight : State or int, optional
        The maximum height of the menu in pixels. Default is None.
        Can be a `State` object for dynamic updates.
    lock : State or threading.Lock, optional
        A threading lock for thread-safe operations. Default is None.
        Can be a `State` object for dynamic updates.
    onClose : State or Callable, optional
        Callback fired when the menu requests to be closed. Default is None.
        Signature: function(event: object, reason: string) => void
        Can be a `State` object for dynamic updates.
    PopoverClasses : State or dict, optional
        Classes applied to the Popover element. Default is None.
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (`backdrop`, `list`, `paper`, `root`, `transition`). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components used for each slot (`backdrop`, `list`, `paper`, `root`, `transition`). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    transitionDuration : State, str, int, dict, or None, optional
        The length of the transition in ms, or 'auto'. Default is 'auto'.
        Can be a `State` object for dynamic updates.
    TransitionProps : State or dict, optional
        Props applied to the transition element. Default is None.
        Deprecated: Use `slotProps.transition` instead.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        The variant to use ("menu" or "selectedMenu"). Default is "selectedMenu".
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QMenu` class,
        supporting props of the native component (e.g., parent, style, className).

    Attributes
    ----------
    VALID_VARIANTS : list[str]
        Valid values for `variant`: ["menu", "selectedMenu"].
    hideSignal : Signal
        Signal emitted when the menu is hidden.

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - The `open` prop is required to control the visibility of the menu.
    - The `MenuListProps` and `TransitionProps` are deprecated; use `slotProps.list` and `slotProps.transition` instead.
    - The `variant` prop affects focus behavior: "menu" prevents item focus, "selectedMenu" focuses the first item.

    Demos:
    - Menu: https://qtmui.com/material-ui/qtmui-menu/

    API Reference:
    - Menu API: https://qtmui.com/material-ui/api/menu/
    """

    VALID_VARIANTS = ["menu", "selectedMenu"]
    hideSignal = Signal()

    def __init__(
        self,
        open: Union[State, bool],
        anchorEl: Optional[Union[State, QWidget, Callable]] = None,
        autoFocus: Union[State, bool] = True,
        children: Optional[Union[State, List[QWidget], QWidget]] = None,
        classes: Optional[Union[State, Dict]] = None,
        disableAutoFocusItem: Union[State, bool] = False,
        id: Optional[Union[State, str]] = None,
        MenuListProps: Optional[Union[State, Dict]] = None,
        minWidth: Optional[Union[State, int]] = None,
        maxWidth: Optional[Union[State, int]] = None,
        maxHeight: Optional[Union[State, int]] = None,
        lock: Optional[Union[State, threading.Lock]] = None,
        onClose: Optional[Union[State, Callable]] = None,
        PopoverClasses: Optional[Union[State, Dict]] = None,
        slotProps: Optional[Union[State, Dict[str, Union[Dict, Callable]]]] = None,
        slots: Optional[Union[State, Dict[str, str]]] = None,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        transitionDuration: Optional[Union[State, str, int, Dict[str, int]]] = "auto",
        TransitionProps: Optional[Union[State, Dict]] = None,
        variant: Union[State, str] = "selectedMenu",
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(f"Menu-{str(uuid.uuid4())}")

        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_open(open)
        self._set_anchorEl(anchorEl)
        self._set_autoFocus(autoFocus)
        self._set_children(children)
        self._set_classes(classes)
        self._set_disableAutoFocusItem(disableAutoFocusItem)
        self._set_id(id)
        self._set_MenuListProps(MenuListProps)
        self._set_minWidth(minWidth)
        self._set_maxWidth(maxWidth)
        self._set_maxHeight(maxHeight)
        self._set_lock(lock)
        self._set_onClose(onClose)
        self._set_PopoverClasses(PopoverClasses)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)
        self._set_transitionDuration(transitionDuration)
        self._set_TransitionProps(TransitionProps)
        self._set_variant(variant)

        self._init_ui()
        self._set_stylesheet()

        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self.destroyed.connect(self._on_destroyed)
        self._connect_signals()

    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.menu", param_name="open", supported_signatures=Union[State, bool])
    def _set_open(self, value):
        """Assign value to open."""
        self._open = value
        if isinstance(value, State):
            value.valueChanged.connect(self._on_open_changed)
        else:
            self._update_visibility()

    def _get_open(self):
        """Get the open value."""
        return self._open.value if isinstance(self._open, State) else self._open

    @_validate_param(file_path="qtmui.material.menu", param_name="anchorEl", supported_signatures=Union[State, QWidget, Callable, type(None)])
    def _set_anchorEl(self, value):
        """Assign value to anchorEl."""
        self._anchorEl = value

    def _get_anchorEl(self):
        """Get the anchorEl value."""
        anchor = self._anchorEl.value if isinstance(self._anchorEl, State) else self._anchorEl
        if callable(anchor):
            anchor = anchor()
        return anchor

    @_validate_param(file_path="qtmui.material.menu", param_name="autoFocus", supported_signatures=Union[State, bool])
    def _set_autoFocus(self, value):
        """Assign value to autoFocus."""
        self._autoFocus = value

    def _get_autoFocus(self):
        """Get the autoFocus value."""
        return self._autoFocus.value if isinstance(self._autoFocus, State) else self._autoFocus

    @_validate_param(file_path="qtmui.material.menu", param_name="children", supported_signatures=Union[State, List, QWidget, type(None)])
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._widget_references.clear()
        self._children = value
        children = value.value if isinstance(value, State) else value

        if isinstance(children, list):
            for child in children:
                if not isinstance(child, QWidget):
                    raise TypeError(f"Each element in children must be a QWidget, got {type(child)}")
                self._widget_references.append(child)
        elif isinstance(children, QWidget):
            self._widget_references.append(children)
        elif children is not None:
            raise TypeError(f"children must be a State, List[QWidget], QWidget, or None, got {type(children)}")

    def _get_children(self):
        """Get the children value."""
        children = self._children.value if isinstance(self._children, State) else self._children
        return children if isinstance(children, list) else [children] if isinstance(children, QWidget) else []

    @_validate_param(file_path="qtmui.material.menu", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.menu", param_name="disableAutoFocusItem", supported_signatures=Union[State, bool])
    def _set_disableAutoFocusItem(self, value):
        """Assign value to disableAutoFocusItem."""
        self._disableAutoFocusItem = value

    def _get_disableAutoFocusItem(self):
        """Get the disableAutoFocusItem value."""
        return self._disableAutoFocusItem.value if isinstance(self._disableAutoFocusItem, State) else self._disableAutoFocusItem

    @_validate_param(file_path="qtmui.material.menu", param_name="id", supported_signatures=Union[State, str, type(None)])
    def _set_id(self, value):
        """Assign value to id."""
        self._id = value

    def _get_id(self):
        """Get the id value."""
        return self._id.value if isinstance(self._id, State) else self._id

    @_validate_param(file_path="qtmui.material.menu", param_name="MenuListProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_MenuListProps(self, value):
        """Assign value to MenuListProps."""
        self._MenuListProps = value or {}

    def _get_MenuListProps(self):
        """Get the MenuListProps value."""
        return self._MenuListProps.value if isinstance(self._MenuListProps, State) else self._MenuListProps

    @_validate_param(file_path="qtmui.material.menu", param_name="minWidth", supported_signatures=Union[State, int, type(None)], validator=lambda x: x > 0 if isinstance(x, int) else True)
    def _set_minWidth(self, value):
        """Assign value to minWidth."""
        self._minWidth = value

    def _get_minWidth(self):
        """Get the minWidth value."""
        return self._minWidth.value if isinstance(self._minWidth, State) else self._minWidth

    @_validate_param(file_path="qtmui.material.menu", param_name="maxWidth", supported_signatures=Union[State, int, type(None)], validator=lambda x: x > 0 if isinstance(x, int) else True)
    def _set_maxWidth(self, value):
        """Assign value to maxWidth."""
        self._maxWidth = value

    def _get_maxWidth(self):
        """Get the maxWidth value."""
        return self._maxWidth.value if isinstance(self._maxWidth, State) else self._maxWidth

    @_validate_param(file_path="qtmui.material.menu", param_name="maxHeight", supported_signatures=Union[State, int, type(None)], validator=lambda x: x > 0 if isinstance(x, int) else True)
    def _set_maxHeight(self, value):
        """Assign value to maxHeight."""
        self._maxHeight = value

    def _get_maxHeight(self):
        """Get the maxHeight value."""
        return self._maxHeight.value if isinstance(self._maxHeight, State) else self._maxHeight

    @_validate_param(file_path="qtmui.material.menu", param_name="lock", supported_signatures=Union[State, threading.Lock, type(None)])
    def _set_lock(self, value):
        """Assign value to lock."""
        self._lock = value

    def _get_lock(self):
        """Get the lock value."""
        return self._lock.value if isinstance(self._lock, State) else self._lock

    @_validate_param(file_path="qtmui.material.menu", param_name="onClose", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClose(self, value):
        """Assign value to onClose."""
        self._onClose = value

    def _get_onClose(self):
        """Get the onClose value."""
        return self._onClose.value if isinstance(self._onClose, State) else self._onClose

    @_validate_param(file_path="qtmui.material.menu", param_name="PopoverClasses", supported_signatures=Union[State, Dict, type(None)])
    def _set_PopoverClasses(self, value):
        """Assign value to PopoverClasses."""
        self._PopoverClasses = value

    def _get_PopoverClasses(self):
        """Get the PopoverClasses value."""
        return self._PopoverClasses.value if isinstance(self._PopoverClasses, State) else self._PopoverClasses

    @_validate_param(file_path="qtmui.material.menu", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        """Assign value to slotProps."""
        self._slotProps = value or {}

    def _get_slotProps(self):
        """Get the slotProps value."""
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.menu", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        """Assign value to slots."""
        self._slots = value or {}

    def _get_slots(self):
        """Get the slots value."""
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.menu", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.menu", param_name="transitionDuration", supported_signatures=Union[State, str, int, Dict, type(None)])
    def _set_transitionDuration(self, value):
        """Assign value to transitionDuration."""
        self._transitionDuration = value

    def _get_transitionDuration(self):
        """Get the transitionDuration value."""
        return self._transitionDuration.value if isinstance(self._transitionDuration, State) else self._transitionDuration

    @_validate_param(file_path="qtmui.material.menu", param_name="TransitionProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_TransitionProps(self, value):
        """Assign value to TransitionProps."""
        self._TransitionProps = value or {}

    def _get_TransitionProps(self):
        """Get the TransitionProps value."""
        return self._TransitionProps.value if isinstance(self._TransitionProps, State) else self._TransitionProps

    @_validate_param(file_path="qtmui.material.menu", param_name="variant", supported_signatures=Union[State, str], valid_values=VALID_VARIANTS)
    def _set_variant(self, value):
        """Assign value to variant."""
        self._variant = value

    def _get_variant(self):
        """Get the variant value."""
        return self._variant.value if isinstance(self._variant, State) else self._variant

    def _init_ui(self):
        """Initialize the UI based on props."""
        self.setWindowFlags(Qt.Popup | Qt.FramelessWindowHint | Qt.NoDropShadowWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.installEventFilter(self)

        # Set layout
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)

        # Create container
        self.container = QFrame()
        self.container.setObjectName("MenuContainer")
        self.container.setLayout(QVBoxLayout())
        self.container.layout().setContentsMargins(3, 3, 3, 3)
        self.container.layout().setSpacing(3)
        self.layout().addWidget(self.container)

        # Apply width constraints
        min_width = self._get_minWidth()
        if min_width:
            self.setMinimumWidth(min_width)
        max_width = self._get_maxWidth()
        if max_width and (not min_width or max_width > min_width):
            self.setMaximumWidth(max_width)

        # Apply children
        self._update_children()

        # Update visibility
        self._update_visibility()

    def _update_children(self):
        """Update the children in the menu."""
        children = self._get_children()
        max_height = self._get_maxHeight()
        slot_props = self._get_slotProps().get("list", {})
        menu_list_props = {**self._get_MenuListProps(), **slot_props}

        # Clear existing layout
        while self.container.layout().count():
            item = self.container.layout().takeAt(0)
            if item.widget():
                item.widget().setParent(None)

        if children:
            if max_height:
                # Use MenuScrollArea for scrollable content
                scroll_area = MenuScrollArea(children=children, **menu_list_props)
                scroll_area.setObjectName("MenuScrollArea")
                self.container.layout().addWidget(scroll_area)
                self.container.layout().setContentsMargins(0, 0, 0, 0)
                if scroll_area.sizeHint().height() < max_height:
                    self.setMinimumHeight(scroll_area.sizeHint().height())
                else:
                    self.setMinimumHeight(max_height)
                self.setMaximumHeight(max_height)
            else:
                # Add children directly
                for child in children:
                    child.setObjectName("MenuItem")
                    self.container.layout().addWidget(child)

    def _update_visibility(self):
        """Update the visibility of the menu based on open prop."""
        if self._get_open():
            anchor = self._get_anchorEl()
            pos = QPoint(0, 0)
            if isinstance(anchor, QWidget):
                pos = anchor.mapToGlobal(anchor.rect().bottomLeft())
            elif isinstance(anchor, QPoint):
                pos = anchor
            self.popup(pos)
            if self._get_autoFocus():
                self._handle_focus()
        else:
            self.hide()

    def _handle_focus(self):
        """Handle focus based on autoFocus and disableAutoFocusItem."""
        if not self._get_autoFocus():
            self.setFocus()
            return
        if self._get_disableAutoFocusItem() or self._get_variant() == "menu":
            self.setFocus()
        else:
            # Focus first focusable item
            for child in self._get_children():
                if child.isEnabled() and child.focusPolicy() != Qt.NoFocus:
                    child.setFocus()
                    break

    def _set_stylesheet(self, component_styled=None):
        """Set the stylesheet for the Menu."""
        self.theme = useTheme()
        component_styled = component_styled or self.theme.components
        menu_styles = component_styled.get("Menu", {}).get("styles", {})
        root_styles = menu_styles.get("root", {})
        paper_styles = menu_styles.get("paper", {})
        root_qss = get_qss_style(root_styles)
        paper_qss = get_qss_style(paper_styles)

        # Handle sx
        sx = self._get_sx()
        sx_qss = ""
        if sx:
            if isinstance(sx, (list, dict)):
                sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
            elif isinstance(sx, Callable):
                sx_result = sx()
                if isinstance(sx_result, (list, dict)):
                    sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
                elif isinstance(sx_result, str):
                    sx_qss = sx_result
            elif isinstance(sx, str) and sx != "":
                sx_qss = sx

        # Handle classes
        classes = self._get_classes()
        classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

        # Handle PopoverClasses
        popover_classes = self._get_PopoverClasses()
        popover_classes_qss = get_qss_style(popover_classes, class_name=f"#{self.container.objectName()}") if popover_classes else ""

        # Apply MUI classes
        mui_classes = ["MuiMenu-root"]
        mui_paper_classes = ["MuiMenu-paper"]

        stylesheet = f"""
            #{self.objectName()} {{
                background-color: transparent;
                border: none;
                {root_qss}
                {classes_qss}
            }}
            #{self.container.objectName()} {{
                {paper_qss}
                {popover_classes_qss}
            }}
            {sx_qss}
        """
        self.setStyleSheet(stylesheet)

    def hideEvent(self, event: QHideEvent):
        """Handle the hide event."""
        self.hideSignal.emit()
        on_close = self._get_onClose()
        if on_close:
            reason = "backdropClick"  # Default reason
            if event.spontaneous():
                reason = "escapeKeyDown"  # Assume escape key for spontaneous hide
            on_close(event, reason)
        super().hideEvent(event)

    def focusInEvent(self, event: QFocusEvent):
        """Handle focus in event."""
        self._handle_focus()
        super().focusInEvent(event)

    def _connect_signals(self):
        """Connect valueChanged signals of State parameters to their slots."""
        if isinstance(self._anchorEl, State):
            self._anchorEl.valueChanged.connect(self._on_anchorEl_changed)
        if isinstance(self._autoFocus, State):
            self._autoFocus.valueChanged.connect(self._on_autoFocus_changed)
        if isinstance(self._children, State):
            self._children.valueChanged.connect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.connect(self._on_classes_changed)
        if isinstance(self._disableAutoFocusItem, State):
            self._disableAutoFocusItem.valueChanged.connect(self._on_disableAutoFocusItem_changed)
        if isinstance(self._id, State):
            self._id.valueChanged.connect(self._on_id_changed)
        if isinstance(self._MenuListProps, State):
            self._MenuListProps.valueChanged.connect(self._on_MenuListProps_changed)
        if isinstance(self._minWidth, State):
            self._minWidth.valueChanged.connect(self._on_minWidth_changed)
        if isinstance(self._maxWidth, State):
            self._maxWidth.valueChanged.connect(self._on_maxWidth_changed)
        if isinstance(self._maxHeight, State):
            self._maxHeight.valueChanged.connect(self._on_maxHeight_changed)
        if isinstance(self._lock, State):
            self._lock.valueChanged.connect(self._on_lock_changed)
        if isinstance(self._onClose, State):
            self._onClose.valueChanged.connect(self._on_onClose_changed)
        if isinstance(self._PopoverClasses, State):
            self._PopoverClasses.valueChanged.connect(self._on_PopoverClasses_changed)
        if isinstance(self._slotProps, State):
            self._slotProps.valueChanged.connect(self._on_slotProps_changed)
        if isinstance(self._slots, State):
            self._slots.valueChanged.connect(self._on_slots_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.connect(self._on_sx_changed)
        if isinstance(self._transitionDuration, State):
            self._transitionDuration.valueChanged.connect(self._on_transitionDuration_changed)
        if isinstance(self._TransitionProps, State):
            self._TransitionProps.valueChanged.connect(self._on_TransitionProps_changed)
        if isinstance(self._variant, State):
            self._variant.valueChanged.connect(self._on_variant_changed)

    def _on_open_changed(self):
        """Handle changes to open."""
        self._update_visibility()

    def _on_anchorEl_changed(self):
        """Handle changes to anchorEl."""
        self._set_anchorEl(self._anchorEl)
        self._update_visibility()

    def _on_autoFocus_changed(self):
        """Handle changes to autoFocus."""
        self._set_autoFocus(self._autoFocus)
        self._handle_focus()

    def _on_children_changed(self):
        """Handle changes to children."""
        self._set_children(self._children)
        self._update_children()

    def _on_classes_changed(self):
        """Handle changes to classes."""
        self._set_classes(self._classes)
        self._set_stylesheet()

    def _on_disableAutoFocusItem_changed(self):
        """Handle changes to disableAutoFocusItem."""
        self._set_disableAutoFocusItem(self._disableAutoFocusItem)
        self._handle_focus()

    def _on_id_changed(self):
        """Handle changes to id."""
        self._set_id(self._id)

    def _on_MenuListProps_changed(self):
        """Handle changes to MenuListProps."""
        self._set_MenuListProps(self._MenuListProps)
        self._update_children()

    def _on_minWidth_changed(self):
        """Handle changes to minWidth."""
        self._set_minWidth(self._minWidth)
        self._init_ui()

    def _on_maxWidth_changed(self):
        """Handle changes to maxWidth."""
        self._set_maxWidth(self._maxWidth)
        self._init_ui()

    def _on_maxHeight_changed(self):
        """Handle changes to maxHeight."""
        self._set_maxHeight(self._maxHeight)
        self._update_children()

    def _on_lock_changed(self):
        """Handle changes to lock."""
        self._set_lock(self._lock)

    def _on_onClose_changed(self):
        """Handle changes to onClose."""
        self._set_onClose(self._onClose)

    def _on_PopoverClasses_changed(self):
        """Handle changes to PopoverClasses."""
        self._set_PopoverClasses(self._PopoverClasses)
        self._set_stylesheet()

    def _on_slotProps_changed(self):
        """Handle changes to slotProps."""
        self._set_slotProps(self._slotProps)
        self._update_children()

    def _on_slots_changed(self):
        """Handle changes to slots."""
        self._set_slots(self._slots)
        self._update_children()

    def _on_sx_changed(self):
        """Handle changes to sx."""
        self._set_sx(self._sx)
        self._set_stylesheet()

    def _on_transitionDuration_changed(self):
        """Handle changes to transitionDuration."""
        self._set_transitionDuration(self._transitionDuration)

    def _on_TransitionProps_changed(self):
        """Handle changes to TransitionProps."""
        self._set_TransitionProps(self._TransitionProps)
        self._update_children()

    def _on_variant_changed(self):
        """Handle changes to variant."""
        self._set_variant(self._variant)
        self._handle_focus()

    def _on_destroyed(self):
        """Clean up connections when the widget is destroyed."""
        if hasattr(self, "theme"):
            self.theme.state.valueChanged.disconnect(self._set_stylesheet)
        if isinstance(self._open, State):
            self._open.valueChanged.disconnect(self._on_open_changed)
        if isinstance(self._anchorEl, State):
            self._anchorEl.valueChanged.disconnect(self._on_anchorEl_changed)
        if isinstance(self._autoFocus, State):
            self._autoFocus.valueChanged.disconnect(self._on_autoFocus_changed)
        if isinstance(self._children, State):
            self._children.valueChanged.disconnect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.disconnect(self._on_classes_changed)
        if isinstance(self._disableAutoFocusItem, State):
            self._disableAutoFocusItem.valueChanged.disconnect(self._on_disableAutoFocusItem_changed)
        if isinstance(self._id, State):
            self._id.valueChanged.disconnect(self._on_id_changed)
        if isinstance(self._MenuListProps, State):
            self._MenuListProps.valueChanged.disconnect(self._on_MenuListProps_changed)
        if isinstance(self._minWidth, State):
            self._minWidth.valueChanged.disconnect(self._on_minWidth_changed)
        if isinstance(self._maxWidth, State):
            self._maxWidth.valueChanged.disconnect(self._on_maxWidth_changed)
        if isinstance(self._maxHeight, State):
            self._maxHeight.valueChanged.disconnect(self._on_maxHeight_changed)
        if isinstance(self._lock, State):
            self._lock.valueChanged.disconnect(self._on_lock_changed)
        if isinstance(self._onClose, State):
            self._onClose.valueChanged.disconnect(self._on_onClose_changed)
        if isinstance(self._PopoverClasses, State):
            self._PopoverClasses.valueChanged.disconnect(self._on_PopoverClasses_changed)
        if isinstance(self._slotProps, State):
            self._slotProps.valueChanged.disconnect(self._on_slotProps_changed)
        if isinstance(self._slots, State):
            self._slots.valueChanged.disconnect(self._on_slots_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.disconnect(self._on_sx_changed)
        if isinstance(self._transitionDuration, State):
            self._transitionDuration.valueChanged.disconnect(self._on_transitionDuration_changed)
        if isinstance(self._TransitionProps, State):
            self._TransitionProps.valueChanged.disconnect(self._on_TransitionProps_changed)
        if isinstance(self._variant, State):
            self._variant.valueChanged.disconnect(self._on_variant_changed)


# coding: utf-8
import uuid
from typing import List, Union, Optional, Dict

from PySide6.QtGui import QAction
from PySide6.QtWidgets import (
    QTableWidget, 
    QHeaderView, 
    QWidget, 
    QTableWidgetItem, 
    QStyledItemDelegate, 
    QApplication, 
    QStyleOptionViewItem,
    QTableView, 
    QTableWidget, 
    QWidget, 
    QTableWidgetItem, 
    QStyle,
    QStyleOptionButton, 
    QFrame, 
    QVBoxLayout,
    QProxyStyle,
    QStyleOption,
    QCheckBox,
    QHBoxLayout,
)
from PySide6.QtCore import (
    Qt, 
    QMargins, 
    QModelIndex, 
    QItemSelectionModel, 
    Property, 
    QRectF, 
    QRect,
)
from PySide6.QtGui import QPainter, QColor, QKeyEvent, QPalette, QBrush, QFont

from typing import TYPE_CHECKING, Callable
if TYPE_CHECKING:
    from .table_row import TableRow

from qtmui.hooks import useState

from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style


from qtmui.material.styles import useTheme
from ....i18n.use_translation import translate, i18n

from ....common.font import getFont
from ....common.style_sheet import isDarkTheme, FluentStyleSheet, themeColor, setCustomStyleSheet
from ...widgets.check_box import CheckBoxIcon
from ...widgets.line_edit import LineEdit
from ...py_iconify import PyIconify
from ...widgets.scroll_bar import SmoothScrollDelegate
from ....qtmui_assets import QTMUI_ASSETS

from ...checkbox import Checkbox
from ...button import Button
from ...box import Box
from ...spacer import HSpacer

class TableItemDelegate(QStyledItemDelegate):

    def __init__(self, parent: QTableView):
        super().__init__(parent)
        self.margin = 2
        self.hoverRow = -1
        self.pressedRow = -1
        self.selectedRows = set()

    def setHoverRow(self, row: int):
        # print('setHoverRow__________', row)
        self.hoverRow = row

    def setPressedRow(self, row: int):
        self.pressedRow = row

    def setSelectedRows(self, indexes: List[QModelIndex]):
        self.selectedRows.clear()
        for index in indexes:
            self.selectedRows.add(index.row())
            if index.row() == self.pressedRow:
                self.pressedRow = -1

    def sizeHint(self, option, index):
        # increase original sizeHint to accommodate space needed for border
        size = super().sizeHint(option, index)
        size = size.grownBy(QMargins(0, self.margin, 0, self.margin))
        return size

    def createEditor(self, parent: QWidget, option: QStyleOptionViewItem, index: QModelIndex) -> QWidget:
        lineEdit = LineEdit(parent)
        lineEdit.setProperty("transparent", False)
        lineEdit.setStyle(QApplication.style())
        lineEdit.setText(option.text)
        lineEdit.setClearButtonEnabled(True)
        return lineEdit



    def updateEditorGeometry(self, editor: QWidget, option: QStyleOptionViewItem, index: QModelIndex):
        rect = option.rect
        y = rect.y() + (rect.height() - editor.height()) // 2
        x, w = max(8, rect.x()), rect.width()
        if index.column() == 0:
            w -= 8

        editor.setGeometry(x, y, w, rect.height())

    def _drawBackground(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
        """ draw row background """
        r = 5
        if index.column() == 0:
            rect = option.rect.adjusted(4, 0, r + 1, 0)
            painter.drawRoundedRect(rect, r, r)
        elif index.column() == index.model().columnCount(index.parent()) - 1:
            rect = option.rect.adjusted(-r - 1, 0, -4, 0)
            painter.drawRoundedRect(rect, r, r)
        else:
            rect = option.rect.adjusted(-1, 0, 1, 0)
            painter.drawRect(rect)

    def _drawIndicator(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
        """ draw indicator """
        y, h = option.rect.y(), option.rect.height()
        ph = round(0.35*h if self.pressedRow == index.row() else 0.257*h)
        painter.setBrush(QBrush(QColor(useTheme().palette.error.main)))
        painter.drawRoundedRect(4, ph + y, 3, h - 2*ph, 1.5, 1.5)

    def initStyleOption(self, option: QStyleOptionViewItem, index: QModelIndex):
        super().initStyleOption(option, index)

        # font
        option.font = index.data(Qt.FontRole) or getFont(13)

        # text color
        textColor = Qt.white if isDarkTheme() else Qt.black
        textBrush = index.data(Qt.ForegroundRole)   # type: QBrush
        if textBrush is not None:
            textColor = textBrush.color()

        option.palette.setColor(QPalette.Text, textColor)
        option.palette.setColor(QPalette.HighlightedText, textColor)

    def paint(self, painter, option, index):
        painter.save()
        painter.setPen(Qt.NoPen)
        painter.setRenderHint(QPainter.Antialiasing)

        # set clipping rect of painter to avoid painting outside the borders
        painter.setClipping(True)
        painter.setClipRect(option.rect)

        # call original paint method where option.rect is adjusted to account for border
        option.rect.adjust(0, self.margin, 0, -self.margin)

        # draw highlight background
        isHover = self.hoverRow == index.row()
        isPressed = self.pressedRow == index.row()
        isAlternate = index.row() % 2 == 0 and self.parent().alternatingRowColors()
        isDark = isDarkTheme()

        c = 255 if isDark else 0
        alpha = 0

        if index.row() not in self.selectedRows:
            if isPressed:
                alpha = 9 if isDark else 6
            elif isHover:
                alpha = 12
            elif isAlternate:
                alpha = 5
        else:
            if isPressed:
                alpha = 15 if isDark else 9
            elif isHover:
                alpha = 25
            else:
                alpha = 17

        if index.data(Qt.ItemDataRole.BackgroundRole):
            painter.setBrush(index.data(Qt.ItemDataRole.BackgroundRole))
        else:
            painter.setBrush(QColor(c, c, c, alpha))

        self._drawBackground(painter, option, index)

        # draw indicator
        if index.row() in self.selectedRows and index.column() == 0 and self.parent().horizontalScrollBar().value() == 0:
            self._drawIndicator(painter, option, index)

        if index.data(Qt.CheckStateRole) is not None:
            self._drawCheckBox(painter, option, index)

        painter.restore()
        super().paint(painter, option, index)

    def _drawCheckBox(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
        painter.save()
        checkState = Qt.CheckState(index.data(Qt.ItemDataRole.CheckStateRole))

        isDark = isDarkTheme()

        r = 4.5
        x = option.rect.x() + 15
        y = option.rect.center().y() - 9.5
        rect = QRectF(x, y, 19, 19)

        if checkState == Qt.CheckState.Unchecked:
            painter.setBrush(QColor(0, 0, 0, 26) if isDark else QColor(0, 0, 0, 6))
            painter.setPen(QColor(255, 255, 255, 142) if isDark else QColor(0, 0, 0, 122))
            painter.drawRoundedRect(rect, r, r)
        else:
            painter.setPen(QColor(useTheme().palette.primary.main))
            painter.setBrush(QColor(useTheme().palette.primary.main))
            painter.drawRoundedRect(rect, r, r)

            if checkState == Qt.CheckState.Checked:
                CheckBoxIcon.ACCEPT.render(painter, rect)
            else:
                CheckBoxIcon.PARTIAL_ACCEPT.render(painter, rect)

        painter.restore()





# import asyncio
# from typing import Optional, Union, Callable, Dict, List, Any
# import uuid
# from PySide6.QtGui import QPainter, QPen, QColor
# from PySide6.QtCore import Qt, Signal, QSize, QTimer
# from PySide6.QtWidgets import QWidget, QVBoxLayout
# from qtmui.material.styles.create_theme.theme_reducer import ThemeState
# from qtmui.material.styles.create_theme.create_palette import PaletteColor
# from ..system.color_manipulator import hex_string_to_qcolor
# from ..button.button import Button
# from qtmui.hooks import State
# from qtmui.material.styles import useTheme
# from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
# from ..utils.validate_params import _validate_param

# class Radio(Button):
#     """
#     A radio button component, styled like Material-UI Radio.

#     The `Radio` component allows users to select one option from a set. It integrates with the `qtmui`
#     framework, retaining existing parameters (except `text` and `hightLight`), adding new parameters,
#     and aligning with MUI Radio props. Inherits from `Button` to support `ButtonBase` props.

#     Parameters
#     ----------
#     id : State or str, optional
#         The id of the input element. Default is None.
#         Can be a `State` object for dynamic updates.
#     checked : State or bool, optional
#         If True, the component is checked. Default is False.
#         Can be a `State` object for dynamic updates.
#     icon : State, QWidget, Callable, or None, optional
#         The icon to display when unchecked. Default is None.
#         Can be a `State` object for dynamic updates or a Callable returning a QWidget.
#     checkedIcon : State, QWidget, Callable, or None, optional
#         The icon to display when checked. Default is None.
#         Can be a `State` object for dynamic updates or a Callable returning a QWidget.
#     color : State or str, optional
#         The color of the component ('default', 'primary', 'secondary', 'error', 'info', 'success', 'warning').
#         Default is 'primary'.
#         Can be a `State` object for dynamic updates.
#     name : State or str, optional
#         Name attribute of the input element. Default is None.
#         Can be a `State` object for dynamic updates.
#     onChange : State or Callable, optional
#         Callback fired when the state changes. Default is None.
#         Can be a `State` object for dynamic updates.
#         Signature: (event: Any, checked: bool) -> None
#     value : State or Any, optional
#         The value of the component. Default is None.
#         Can be a `State` object for dynamic updates.
#     classes : State or dict, optional
#         Override or extend the styles applied to the component. Default is None.
#         Can be a `State` object for dynamic updates.
#     disabled : State or bool, optional
#         If True, the component is disabled. Default is False.
#         Can be a `State` object for dynamic updates.
#     disableRipple : State or bool, optional
#         If True, the ripple effect is disabled. Default is False.
#         Can be a `State` object for dynamic updates.
#     inputProps : State or dict, optional
#         Attributes applied to the input element. Default is None.
#         Can be a `State` object for dynamic updates.
#     inputRef : State or Callable, optional
#         Ref to the input element. Default is None.
#         Can be a `State` object for dynamic updates.
#     required : State or bool, optional
#         If True, the input element is required. Default is False.
#         Can be a `State` object for dynamic updates.
#     size : State or str, optional
#         The size of the component ('small', 'medium'). Default is 'medium'.
#         Can be a `State` object for dynamic updates.
#     slotProps : State or dict, optional
#         Props for slots ({input, root}). Default is {}.
#         Can be a `State` object for dynamic updates.
#     slots : State or dict, optional
#         Components for slots ({input, root}). Default is {}.
#         Can be a `State` object for dynamic updates.
#     sx : State, list, dict, Callable, str, or None, optional
#         System prop for CSS overrides and additional styles. Default is None.
#         Can be a `State` object for dynamic updates.
#     **kwargs
#         Additional keyword arguments passed to the parent `Button` class,
#         supporting props of the `ButtonBase` component (e.g., disableElevation).

#     Signals
#     -------
#     changed : Signal
#         Emitted when the checked state or value changes.

#     Notes
#     -----
#     - Existing parameters `text` and `hightLight` are removed; all other parameters are retained.
#     - New parameters added to align with MUI: `classes`, `disabled`, `disableRipple`, `inputProps`,
#       `inputRef`, `required`, `size`, `slotProps`, `slots`, `sx`.
#     - Props of the `ButtonBase` component are supported via `**kwargs`.
#     - MUI classes applied: `MuiRadio-root`, `Mui-checked`, `Mui-disabled`.
#     - Integrates with `Button` for `ButtonBase` props and supports custom icons via `QWidget`.

#     Demos:
#     - Radio: https://qtmui.com/material-ui/qtmui-radio/

#     API Reference:
#     - Radio API: https://qtmui.com/material-ui/api/radio/
#     """

#     changed = Signal(object, bool)

#     VALID_COLORS = ['default', 'primary', 'secondary', 'error', 'info', 'success', 'warning']
#     VALID_SIZES = ['small', 'medium']

#     def __init__(
#         self,
#         id: Optional[Union[State, str]] = None,
#         checked: Union[State, bool] = False,
#         icon: Optional[Union[State, QWidget, Callable]] = None,
#         checkedIcon: Optional[Union[State, QWidget, Callable]] = None,
#         color: Union[State, str] = 'primary',
#         name: Optional[Union[State, str]] = None,
#         onChange: Optional[Union[State, Callable]] = None,
#         value: Optional[Union[State, Any]] = None,
#         classes: Optional[Union[State, Dict]] = None,
#         disabled: Union[State, bool] = False,
#         disableRipple: Union[State, bool] = False,
#         inputProps: Optional[Union[State, Dict]] = None,
#         inputRef: Optional[Union[State, Callable]] = None,
#         required: Union[State, bool] = False,
#         size: Union[State, str] = 'medium',
#         slotProps: Union[State, Dict] = {},
#         slots: Union[State, Dict] = {},
#         sx: Optional[Union[State, List, Dict, Callable, str]] = None,
#         *args,
#         **kwargs
#     ):
#         super().__init__(variant="text", *args, **kwargs)
#         self.setObjectName(f"Radio-{str(uuid.uuid4())}")

#         self.theme = useTheme()
#         self._widget_references = []
#         self._indicator_color_checked = None

#         # Set properties with validation
#         self._set_id(id)
#         self._set_checked(checked)
#         self._set_icon(icon)
#         self._set_checkedIcon(checkedIcon)
#         self._set_color(color)
#         self._set_name(name)
#         self._set_onChange(onChange)
#         self._set_value(value)
#         self._set_classes(classes)
#         self._set_disabled(disabled)
#         self._set_disableRipple(disableRipple)
#         self._set_inputProps(inputProps)
#         self._set_inputRef(inputRef)
#         self._set_required(required)
#         self._set_size(size)
#         self._set_slotProps(slotProps)
#         self._set_slots(slots)
#         self._set_sx(sx)

#         self._init_ui()
#         self._set_stylesheet()

#         self.useEffect(
        #     self._set_stylesheet,
        #     [theme.state]
        # )
#         self.destroyed.connect(self._on_destroyed)
#         self._connect_signals()

#     # Setter and Getter methods
#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="id",
#         supported_signatures=Union[State, str, type(None)]
#     )
#     def _set_id(self, value):
#         """Assign value to id."""
#         self._id = value
#         if value and not isinstance(value, State):
#             self.setObjectName(value)

#     def _get_id(self):
#         """Get the id value."""
#         return self._id.value if isinstance(self._id, State) else self._id

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="checked",
#         supported_signatures=Union[State, bool]
#     )
#     def _set_checked(self, value):
#         """Assign value to checked."""
#         self._checked = value
#         self._checkedValue = value.value if isinstance(value, State) else value

#     def _get_checked(self):
#         """Get the checked value."""
#         return self._checked.value if isinstance(self._checked, State) else self._checked

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="icon",
#         supported_signatures=Union[State, QWidget, Callable, type(None)]
#     )
#     def _set_icon(self, value):
#         """Assign value to icon."""
#         self._icon = value

#     def _get_icon(self):
#         """Get the icon value."""
#         icon = self._icon
#         if isinstance(icon, State):
#             icon = icon.value
#         if callable(icon):
#             icon = icon()
#         return icon if isinstance(icon, QWidget) else None

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="checkedIcon",
#         supported_signatures=Union[State, QWidget, Callable, type(None)]
#     )
#     def _set_checkedIcon(self, value):
#         """Assign value to checkedIcon."""
#         self._checkedIcon = value

#     def _get_checkedIcon(self):
#         """Get the checkedIcon value."""
#         icon = self._checkedIcon
#         if isinstance(icon, State):
#             icon = icon.value
#         if callable(icon):
#             icon = icon()
#         return icon if isinstance(icon, QWidget) else None

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="color",
#         supported_signatures=Union[State, str],
#         valid_values=VALID_COLORS
#     )
#     def _set_color(self, value):
#         """Assign value to color."""
#         self._color = value

#     def _get_color(self):
#         """Get the color value."""
#         return self._color.value if isinstance(self._color, State) else self._color

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="name",
#         supported_signatures=Union[State, str, type(None)]
#     )
#     def _set_name(self, value):
#         """Assign value to name."""
#         self._name = value

#     def _get_name(self):
#         """Get the name value."""
#         return self._name.value if isinstance(self._name, State) else self._name

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="onChange",
#         supported_signatures=Union[State, Callable, type(None)]
#     )
#     def _set_onChange(self, value):
#         """Assign value to onChange."""
#         self._onChange = value

#     def _get_onChange(self):
#         """Get the onChange value."""
#         return self._onChange.value if isinstance(self._onChange, State) else self._onChange

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="value",
#         supported_signatures=Union[State, Any, type(None)]
#     )
#     def _set_value(self, value):
#         """Assign value to value."""
#         self._value = value

#     def _get_value(self):
#         """Get the value value."""
#         return self._value.value if isinstance(self._value, State) else self._value

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="classes",
#         supported_signatures=Union[State, Dict, type(None)]
#     )
#     def _set_classes(self, value):
#         """Assign value to classes."""
#         self._classes = value

#     def _get_classes(self):
#         """Get the classes value."""
#         return self._classes.value if isinstance(self._classes, State) else self._classes

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="disabled",
#         supported_signatures=Union[State, bool]
#     )
#     def _set_disabled(self, value):
#         """Assign value to disabled."""
#         self._disabled = value
#         self.setEnabled(not (value.value if isinstance(value, State) else value))

#     def _get_disabled(self):
#         """Get the disabled value."""
#         return self._disabled.value if isinstance(self._disabled, State) else self._disabled

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="disableRipple",
#         supported_signatures=Union[State, bool]
#     )
#     def _set_disableRipple(self, value):
#         """Assign value to disableRipple."""
#         self._disableRipple = value

#     def _get_disableRipple(self):
#         """Get the disableRipple value."""
#         return self._disableRipple.value if isinstance(self._disableRipple, State) else self._disableRipple

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="inputProps",
#         supported_signatures=Union[State, Dict, type(None)]
#     )
#     def _set_inputProps(self, value):
#         """Assign value to inputProps."""
#         self._inputProps = value

#     def _get_inputProps(self):
#         """Get the inputProps value."""
#         return self._inputProps.value if isinstance(self._inputProps, State) else self._inputProps

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="inputRef",
#         supported_signatures=Union[State, Callable, type(None)]
#     )
#     def _set_inputRef(self, value):
#         """Assign value to inputRef."""
#         self._inputRef = value

#     def _get_inputRef(self):
#         """Get the inputRef value."""
#         return self._inputRef.value if isinstance(self._inputRef, State) else self._inputRef

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="required",
#         supported_signatures=Union[State, bool]
#     )
#     def _set_required(self, value):
#         """Assign value to required."""
#         self._required = value

#     def _get_required(self):
#         """Get the required value."""
#         return self._required.value if isinstance(self._required, State) else self._required

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="size",
#         supported_signatures=Union[State, str],
#         valid_values=VALID_SIZES
#     )
#     def _set_size(self, value):
#         """Assign value to size."""
#         self._size = value

#     def _get_size(self):
#         """Get the size value."""
#         return self._size.value if isinstance(self._size, State) else self._size

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="slotProps",
#         supported_signatures=Union[State, Dict]
#     )
#     def _set_slotProps(self, value):
#         """Assign value to slotProps."""
#         self._slotProps = value

#     def _get_slotProps(self):
#         """Get the slotProps value."""
#         return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="slots",
#         supported_signatures=Union[State, Dict]
#     )
#     def _set_slots(self, value):
#         """Assign value to slots."""
#         self._slots = value

#     def _get_slots(self):
#         """Get the slots value."""
#         return self._slots.value if isinstance(self._slots, State) else self._slots

#     @_validate_param(
#         file_path="qtmui.material.radio",
#         param_name="sx",
#         supported_signatures=Union[State, List, Dict, Callable, str, type(None)]
#     )
#     def _set_sx(self, value):
#         """Assign value to sx."""
#         self._sx = value

#     def _get_sx(self):
#         """Get the sx value."""
#         return self._sx.value if isinstance(self._sx, State) else self._sx

#     def _init_ui(self):
#         """Initialize the UI based on props."""
#         self.setMouseTracking(True)
#         self.setLayout(QVBoxLayout())
#         self.layout().setContentsMargins(0, 0, 0, 0)
#         self.layout().setSpacing(0)

#         # Clear previous widgets
#         self._widget_references.clear()
#         while self.layout().count():
#             item = self.layout().takeAt(0)
#             if item.widget():
#                 item.widget().setParent(None)

#         # Set size
#         size = self._get_size()
#         self.setFixedSize(QSize(30, 30) if size == "small" else QSize(36, 36))

#         # Add icon
#         icon = self._get_checkedIcon() if self._get_checked() else self._get_icon()
#         if icon:
#             self.layout().addWidget(icon)
#             self._widget_references.append(icon)
#         else:
#             # Default radio rendering will be handled in paintEvent
#             pass

#         # Connect click event
#         self.clicked.connect(self._on_click)

#         # Apply disabled state
#         self.setEnabled(not self._get_disabled())

#         # Apply inputRef
#         input_ref = self._get_inputRef()
#         if input_ref:
#             input_ref(self)

#     def _on_click(self):
#         """Handle click event."""
#         if self._get_disabled():
#             return
#         self._checkedValue = not self._checkedValue
#         self._set_checked(self._checkedValue)
#         self.changed.emit(self._get_value(), self._checkedValue)
#         if self._get_onChange():
#             self._get_onChange()(None, self._checkedValue)
#         self._init_ui()  # Update icon
#         self.update()

#     def _set_stylesheet(self, component_styled=None):
#         """Set the stylesheet for the Radio."""
#         super()._set_stylesheet(component_styled)
#         self.theme = useTheme()
#         component_styled = component_styled or self.theme.components
#         radio_styles = component_styled.get("Radio", {}).get("styles", {})
#         root_styles = radio_styles.get("root", {})
#         root_qss = get_qss_style(root_styles)

#         # Determine color
#         color = self._get_color()
#         palette_color = self.theme.palette.get(color, self.theme.palette.primary)
#         self._indicator_color_checked = hex_string_to_qcolor(palette_color.main)

#         # Handle sx
#         sx = self._get_sx()
#         sx_qss = ""
#         if sx:
#             if isinstance(sx, (list, dict)):
#                 sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
#             elif isinstance(sx, Callable):
#                 sx_result = sx()
#                 if isinstance(sx_result, (list, dict)):
#                     sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
#                 elif isinstance(sx_result, str):
#                     sx_qss = sx_result
#             elif isinstance(sx, str) and sx != "":
#                 sx_qss = sx

#         # Handle classes
#         classes = self._get_classes()
#         classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

#         # Handle slotProps.root
#         root_props = self._get_slotProps().get('root', {})
#         root_props_qss = get_qss_style(root_props.get('sx', {}), class_name=f"#{self.objectName()}")

#         # Apply MUI classes
#         mui_classes = ["MuiRadio-root"]
#         if self._get_checked():
#             mui_classes.append("Mui-checked")
#         if self._get_disabled():
#             mui_classes.append("Mui-disabled")

#         stylesheet = f"""
#             #{self.objectName()} {{
#                 {root_qss}
#                 {classes_qss}
#                 {root_props_qss}
#                 background: transparent;
#                 border-radius: {"15px" if self._get_size() == "small" else "18px"};
#             }}
#             {sx_qss}
#         """
#         self.setStyleSheet(stylesheet)

#     def paintEvent(self, event):
#         """Custom paint event for default radio rendering."""
#         super().paintEvent(event)
#         if self._get_icon() or self._get_checkedIcon():
#             return  # Skip default rendering if custom icons are provided

#         painter = QPainter(self)
#         painter.setRenderHint(QPainter.Antialiasing)

#         size = self._get_size()
#         padding = 6 if size == "small" else 8
#         border = 2 if self._get_checked() else 1

#         rect = self.rect().adjusted(padding, padding, -padding, -padding)
#         height = rect.height()
#         center_point = rect.x()
#         circle_diameter = height

#         if self._get_checked():
#             painter.setBrush(Qt.NoBrush)
#             painter.setPen(QPen(self._indicator_color_checked, border))
#             painter.drawEllipse(center_point - 1, center_point, circle_diameter, circle_diameter)

#             inner_padding = padding + 4
#             inner_rect = self.rect().adjusted(inner_padding, inner_padding, -inner_padding, -inner_padding)
#             inner_height = inner_rect.height()
#             inner_center = inner_rect.x()
#             inner_diameter = inner_height
#             painter.setPen(Qt.NoPen)
#             painter.setBrush(self._indicator_color_checked)
#             painter.drawEllipse(inner_center - 1, inner_center, inner_diameter, inner_diameter)
#         else:
#             painter.setBrush(Qt.NoBrush)
#             painter.setPen(QPen(hex_string_to_qcolor(self.theme.palette.text.secondary), border))
#             painter.drawEllipse(center_point - 1, center_point, circle_diameter, circle_diameter)

#         painter.end()

#     def _connect_signals(self):
#         """Connect valueChanged signals of State parameters to their slots."""
#         if isinstance(self._id, State):
#             self._id.valueChanged.connect(self._on_id_changed)
#         if isinstance(self._checked, State):
#             self._checked.valueChanged.connect(self._on_checked_changed)
#         if isinstance(self._icon, State):
#             self._icon.valueChanged.connect(self._on_icon_changed)
#         if isinstance(self._checkedIcon, State):
#             self._checkedIcon.valueChanged.connect(self._on_checkedIcon_changed)
#         if isinstance(self._color, State):
#             self._color.valueChanged.connect(self._on_color_changed)
#         if isinstance(self._name, State):
#             self._name.valueChanged.connect(self._on_name_changed)
#         if isinstance(self._onChange, State):
#             self._onChange.valueChanged.connect(self._on_onChange_changed)
#         if isinstance(self._value, State):
#             self._value.valueChanged.connect(self._on_value_changed)
#         if isinstance(self._classes, State):
#             self._classes.valueChanged.connect(self._on_classes_changed)
#         if isinstance(self._disabled, State):
#             self._disabled.valueChanged.connect(self._on_disabled_changed)
#         if isinstance(self._disableRipple, State):
#             self._disableRipple.valueChanged.connect(self._on_disableRipple_changed)
#         if isinstance(self._inputProps, State):
#             self._inputProps.valueChanged.connect(self._on_inputProps_changed)
#         if isinstance(self._inputRef, State):
#             self._inputRef.valueChanged.connect(self._on_inputRef_changed)
#         if isinstance(self._required, State):
#             self._required.valueChanged.connect(self._on_required_changed)
#         if isinstance(self._size, State):
#             self._size.valueChanged.connect(self._on_size_changed)
#         if isinstance(self._slotProps, State):
#             self._slotProps.valueChanged.connect(self._on_slotProps_changed)
#         if isinstance(self._slots, State):
#             self._slots.valueChanged.connect(self._on_slots_changed)
#         if isinstance(self._sx, State):
#             self._sx.valueChanged.connect(self._on_sx_changed)

#     def _on_id_changed(self):
#         """Handle changes to id."""
#         self._set_id(self._id)

#     def _on_checked_changed(self):
#         """Handle changes to checked."""
#         self._set_checked(self._checked)
#         self._init_ui()
#         self.update()

#     def _on_icon_changed(self):
#         """Handle changes to icon."""
#         self._set_icon(self._icon)
#         self._init_ui()

#     def _on_checkedIcon_changed(self):
#         """Handle changes to checkedIcon."""
#         self._set_checkedIcon(self._checkedIcon)
#         self._init_ui()

#     def _on_color_changed(self):
#         """Handle changes to color."""
#         self._set_color(self._color)
#         self._set_stylesheet()

#     def _on_name_changed(self):
#         """Handle changes to name."""
#         self._set_name(self._name)

#     def _on_onChange_changed(self):
#         """Handle changes to onChange."""
#         self._set_onChange(self._onChange)

#     def _on_value_changed(self):
#         """Handle changes to value."""
#         self._set_value(self._value)

#     def _on_classes_changed(self):
#         """Handle changes to classes."""
#         self._set_classes(self._classes)
#         self._set_stylesheet()

#     def _on_disabled_changed(self):
#         """Handle changes to disabled."""
#         self._set_disabled(self._disabled)
#         self._set_stylesheet()

#     def _on_disableRipple_changed(self):
#         """Handle changes to disableRipple."""
#         self._set_disableRipple(self._disableRipple)

#     def _on_inputProps_changed(self):
#         """Handle changes to inputProps."""
#         self._set_inputProps(self._inputProps)

#     def _on_inputRef_changed(self):
#         """Handle changes to inputRef."""
#         self._set_inputRef(self._inputRef)

#     def _on_required_changed(self):
#         """Handle changes to required."""
#         self._set_required(self._required)

#     def _on_size_changed(self):
#         """Handle changes to size."""
#         self._set_size(self._size)
#         self._init_ui()

#     def _on_slotProps_changed(self):
#         """Handle changes to slotProps."""
#         self._set_slotProps(self._slotProps)
#         self._set_stylesheet()

#     def _on_slots_changed(self):
#         """Handle changes to slots."""
#         self._set_slots(self._slots)
#         self._init_ui()

#     def _on_sx_changed(self):
#         """Handle changes to sx."""
#         self._set_sx(self._sx)
#         self._set_stylesheet()

#     def _on_destroyed(self):
#         """Clean up connections when the widget is destroyed."""
#         if hasattr(self, "theme"):
#             self.theme.state.valueChanged.disconnect(self._set_stylesheet)
#         if isinstance(self._id, State):
#             self._id.valueChanged.disconnect(self._on_id_changed)
#         if isinstance(self._checked, State):
#             self._checked.valueChanged.disconnect(self._on_checked_changed)
#         if isinstance(self._icon, State):
#             self._icon.valueChanged.disconnect(self._on_icon_changed)
#         if isinstance(self._checkedIcon, State):
#             self._checkedIcon.valueChanged.disconnect(self._on_checkedIcon_changed)
#         if isinstance(self._color, State):
#             self._color.valueChanged.disconnect(self._on_color_changed)
#         if isinstance(self._name, State):
#             self._name.valueChanged.disconnect(self._on_name_changed)
#         if isinstance(self._onChange, State):
#             self._onChange.valueChanged.disconnect(self._on_onChange_changed)
#         if isinstance(self._value, State):
#             self._value.valueChanged.disconnect(self._on_value_changed)
#         if isinstance(self._classes, State):
#             self._classes.valueChanged.disconnect(self._on_classes_changed)
#         if isinstance(self._disabled, State):
#             self._disabled.valueChanged.disconnect(self._on_disabled_changed)
#         if isinstance(self._disableRipple, State):
#             self._disableRipple.valueChanged.disconnect(self._on_disableRipple_changed)
#         if isinstance(self._inputProps, State):
#             self._inputProps.valueChanged.disconnect(self._on_inputProps_changed)
#         if isinstance(self._inputRef, State):
#             self._inputRef.valueChanged.disconnect(self._on_inputRef_changed)
#         if isinstance(self._required, State):
#             self._required.valueChanged.disconnect(self._on_required_changed)
#         if isinstance(self._size, State):
#             self._size.valueChanged.disconnect(self._on_size_changed)
#         if isinstance(self._slotProps, State):
#             self._slotProps.valueChanged.disconnect(self._on_slotProps_changed)
#         if isinstance(self._slots, State):
#             self._slots.valueChanged.disconnect(self._on_slots_changed)
#         if isinstance(self._sx, State):
#             self._sx.valueChanged.disconnect(self._on_sx_changed)